@@ -32,6 +32,7 @@ Magick::Pixels::Pixels( Magick::Image &image_ )
 
   if (!_view)
     _image.throwImageException();
+  (void) DestroyExceptionInfo( &_exception );
 }
 
 // Destroy pixel view
@@ -40,7 +41,6 @@ Magick::Pixels::~Pixels( void )
   if ( _view )
     _view = DestroyCacheView( _view );
   
-  DestroyExceptionInfo( &_exception );
 }
 
 // Transfer pixels from the image to the pixel view as defined by
@@ -115,7 +115,7 @@ Magick::PixelPacket* Magick::Pixels::set ( const int x_,
 // Return pixel colormap index array
 Magick::IndexPacket* Magick::Pixels::indexes ( void )
 {
-  IndexPacket* pixel_indexes = GetCacheViewAuthenticIndexes( _view );
+  IndexPacket* pixel_indexes = GetCacheViewAuthenticIndexQueue( _view );
 
   if ( !pixel_indexes )
     _image.throwImageException();
