@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -433,49 +433,10 @@ get_page_image(LoadContext *lc, ddjvu_page_t *page, int x, int y, int w, int h,
 }
 
 
-static int
-get_page_line(PixelPacket *q, LoadContext *lc, int row)
-{
-
-        ddjvu_rect_t rect, pagerect;
-        rect.x = 0;
-        rect.y = row;
-        rect.h = 1;             /* /10 */
-
-        pagerect.x = 0;
-        pagerect.y = 0;
-        pagerect.w = lc->image->columns;
-        pagerect.h = lc->image->rows;
-
-
-        ddjvu_format_t *format = ddjvu_format_create(
-#if RGB
-                DDJVU_FORMAT_RGB24
-#else
-                DDJVU_FORMAT_GREY8
-#endif
-                ,
-                0, NULL);
-        ddjvu_format_set_row_order(format, 0);
-        ddjvu_format_set_y_direction(format, 1);
-
-        int ret = ddjvu_page_render(lc->page,
-                                    DDJVU_RENDER_COLOR, /* ddjvu_render_mode_t */
-                                    &pagerect,
-                                    &rect,     /* mmc: ?? */
-                                    format,
-                                    pagerect.w
-#if RGB
-                                    * 3
-#endif
-                                    , /* ?? */
-                                    (char*)q);
-        ddjvu_format_release(format);
-        return ret;
-}
+#if defined(MAGICKCORE_DJVU_DELEGATE)
 
 static int
-get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
+get_page_line(LoadContext *lc, int row, QuantumInfo* quantum_info)
 {
 
         ddjvu_rect_t rect, pagerect;
@@ -528,6 +489,7 @@ get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
         ddjvu_format_release(format);
         return ret;
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -639,6 +601,8 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
 
 #if 1                           /* per_line */
         quantum_info=AcquireQuantumInfo(image_info,image);
+        if (quantum_info == (QuantumInfo *) NULL)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
 
         /* q = QueueAuthenticPixels(image,0,0,image->columns,image->rows); */
         get_page_image(lc, lc->page, 0, 0, info.width, info.height, quantum_info);
@@ -647,15 +611,9 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
         for (i = 0;i< image->rows; i++)
                 {
                         printf("%d\n",i);
-#if 0
                         q = QueueAuthenticPixels(image,0,i,image->columns,1);
-                        get_page_line(q,lc, i);
+                        get_page_line(lc, i, quantum_info);
                         SyncAuthenticPixels(image);
-#else
-                        q = QueueAuthenticPixels(image,0,i,image->columns,1);
-                        get_page_line_new(lc, i, quantum_info);
-                        SyncAuthenticPixels(image);
-#endif
                 }
 
 #endif /* per_line */
