@@ -1004,11 +1004,6 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Convert DPX raster image to pixel packets.
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   samples_per_pixel=1;
   quantum_type=GrayQuantum;
   switch (component_type)
@@ -1016,6 +1011,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     case CbYCrY422ComponentType:
     {
       samples_per_pixel=2;
+      quantum_type=CbYCrYQuantum;
       break;
     }
     case CbYACrYA4224ComponentType:
@@ -1039,6 +1035,8 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   switch (component_type)
   {
+    case CbYCrY422ComponentType:
+    case CbYACrYA4224ComponentType:
     case CbYCr444ComponentType:
     {
       image->colorspace=Rec709YCbCrColorspace;
@@ -1052,13 +1050,13 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     default:
     {
       image->colorspace=RGBColorspace;
+      if (dpx.image.image_element[0].transfer == LogarithmicColorimetric)
+        image->colorspace=LogColorspace;
       if (dpx.image.image_element[0].transfer == PrintingDensityColorimetric)
         image->colorspace=LogColorspace;
       break;
     }
   }
-  if (image->colorspace != UndefinedColorspace)
-    image->colorspace=image->colorspace;
   extent=GetBytesPerRow(image->columns,samples_per_pixel,image->depth,
     dpx.image.image_element[0].packing == 0 ? MagickFalse : MagickTrue);
   if ((quantum_type == GrayQuantum) &&
@@ -1081,6 +1079,8 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         pixel;
 
       quantum_info=AcquireQuantumInfo(image_info,image);
+      if (quantum_info == (QuantumInfo *) NULL)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       SetQuantumQuantum(quantum_info,32);
       SetQuantumPack(quantum_info,dpx.image.image_element[0].packing == 0 ?
         MagickTrue : MagickFalse);
@@ -1134,26 +1134,23 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         row;
 
       QuantumInfo
-        **quantum_info;
+        *quantum_info;
 
       ViewInfo
-        **image_view;
+        *image_view;
 
       /*
         DPX any-bit pixel format.
       */
       status=MagickTrue;
       row=0;
-      quantum_info=AcquireQuantumInfoThreadSet(image_info,image);
-      if (quantum_info == (QuantumInfo **) NULL)
+      quantum_info=AcquireQuantumInfo(image_info,image);
+      if (quantum_info == (QuantumInfo *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
-      {
-        SetQuantumQuantum(quantum_info[i],32);
-        SetQuantumPack(quantum_info[i],dpx.image.image_element[0].packing == 0 ?
-          MagickTrue : MagickFalse);
-      }
-      image_view=AcquireCacheViewThreadSet(image);
+      SetQuantumQuantum(quantum_info,32);
+      SetQuantumPack(quantum_info,dpx.image.image_element[0].packing == 0 ?
+        MagickTrue : MagickFalse);
+      image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,1) shared(row,status,quantum_type)
 #endif
@@ -1165,9 +1162,6 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         MagickBooleanType
           sync;
 
-        register long
-          id;
-
         register PixelPacket
           *q;
 
@@ -1182,8 +1176,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        pixels=GetQuantumPixels(quantum_info[id]);
+        pixels=GetQuantumPixels(quantum_info);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp critical
 #endif
@@ -1203,21 +1196,21 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (count != (ssize_t) extent)
           status=MagickFalse;
-        q=QueueCacheViewAuthenticPixels(image_view[id],0,offset,image->columns,
+        q=QueueCacheViewAuthenticPixels(image_view,0,offset,image->columns,
           1,exception);
         if (q == (PixelPacket *) NULL)
           {
             status=MagickFalse;
             continue;
           }
-        length=ImportQuantumPixels(image,image_view[id],quantum_info[id],
-          quantum_type,pixels,exception);
-        sync=SyncCacheViewAuthenticPixels(image_view[id],exception);
+        length=ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
+          pixels,exception);
+        sync=SyncCacheViewAuthenticPixels(image_view,exception);
         if (sync == MagickFalse)
           status=MagickFalse;
       }
-      image_view=DestroyCacheViewThreadSet(image_view);
-      quantum_info=DestroyQuantumInfoThreadSet(quantum_info);
+      image_view=DestroyCacheView(image_view);
+      quantum_info=DestroyQuantumInfo(quantum_info);
       if (status == MagickFalse)
         ThrowReaderException(CorruptImageError,"UnableToReadImageData");
     }
