@@ -17,7 +17,7 @@
 %                                 July 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -138,7 +138,6 @@ static MagickBooleanType IsMETA(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 #define BUFFER_SZ 4096
 
@@ -151,7 +150,6 @@ typedef struct _html_code
     val;
 } html_code;
 
-#include "magick/quantum-private.h"
 static html_code html_codes[] = {
 #ifdef HANDLE_GT_LT
   { 4,"&lt;",'<' },
@@ -1100,7 +1098,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -1116,7 +1114,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       /*
         Read 8BIM binary metadata.
       */
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=(unsigned char *) AcquireQuantumMemory(length,sizeof(unsigned char));
@@ -1164,7 +1162,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         name[MaxTextExtent];
 
       (void) FormatMagickString(name,MaxTextExtent,"APP%d",1);
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=(unsigned char *) AcquireQuantumMemory(length,sizeof(unsigned char));
@@ -1190,7 +1188,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
               ThrowReaderException(CoderError,"NoIPTCProfileAvailable");
             }
           profile=CloneStringInfo((StringInfo *) image_info->profile);
-          iptc=AllocateImage((ImageInfo *) NULL);
+          iptc=AcquireImage((ImageInfo *) NULL);
           if (iptc == (Image *) NULL)
             {
               blob=DetachBlob(buff->blob);
@@ -1234,12 +1232,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             count,
             length;
 
-          buffer=(unsigned char *) AcquireQuantumMemory(MagickMaxBufferSize,
+          buffer=(unsigned char *) AcquireQuantumMemory(MagickMaxBufferExtent,
             sizeof(*buffer));
           if (buffer != (unsigned char *) NULL)
             {
               i=0;
-              while ((length=ReadBlob(image,MagickMaxBufferSize,buffer)) != 0)
+              while ((length=ReadBlob(image,MagickMaxBufferExtent,buffer)) != 0)
               {
                 count=0;
                 for (i=0; i < (long) length; i+=count)
@@ -1268,7 +1266,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   if ((LocaleCompare(image_info->magick,"ICC") == 0) ||
       (LocaleCompare(image_info->magick,"ICM") == 0))
     {
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=(unsigned char *) AcquireQuantumMemory(length,sizeof(unsigned char));
@@ -1298,7 +1296,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       register unsigned char
         *p;
 
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=(unsigned char *) AcquireQuantumMemory(length,sizeof(unsigned char));
@@ -1339,7 +1337,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       p[10]=(unsigned char) (length >> 8);
       p[11]=(unsigned char) (length & 0xff);
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
-      (void) SetImageProfile(image,"iptc",profile);
+      (void) SetImageProfile(image,"8bim",profile);
       profile=DestroyStringInfo(profile);
       blob=DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
@@ -1347,7 +1345,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
     }
   if (LocaleCompare(image_info->magick,"XMP") == 0)
     {
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=(unsigned char *) AcquireQuantumMemory(length,sizeof(unsigned char));
@@ -1372,7 +1370,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -1588,7 +1586,6 @@ ModuleExport void UnregisterMETAImage(void)
 %
 %    o image: A pointer to a Image structure.
 %
-%
 */
 
 static size_t GetIPTCStream(unsigned char **info,unsigned long length)
@@ -1900,7 +1897,7 @@ static int formatIPTC(Image *ifile, Image *ofile)
         taglen = (c << 8) | c0;
       }
     if (taglen < 0) return -1;
-    /* make a buffer to hold the tag data and snag it from the input stream */
+    /* make a buffer to hold the tag datand snag it from the input stream */
     str=(unsigned char *) AcquireQuantumMemory((size_t) (taglen+MaxTextExtent),
       sizeof(*str));
     if (str == (unsigned char *) NULL)
@@ -2030,7 +2027,7 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
       }
     if (taglen < 0)
       return(-1);
-    /* make a buffer to hold the tag data and snag it from the input stream */
+    /* make a buffer to hold the tag datand snag it from the input stream */
     str=(unsigned char *) AcquireQuantumMemory((size_t) (taglen+MaxTextExtent),
       sizeof(*str));
     if (str == (unsigned char *) NULL)
@@ -2150,7 +2147,7 @@ static int format8BIM(Image *ifile, Image *ofile)
     }
     count = (ssize_t) ReadBlobMSBLong(ifile);
     if (count < 0) return -1;
-    /* make a buffer to hold the data and snag it from the input stream */
+    /* make a buffer to hold the datand snag it from the input stream */
     str=(unsigned char *) AcquireQuantumMemory((size_t) count,sizeof(*str));
     if (str == (unsigned char *) NULL)
       {
@@ -2230,7 +2227,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
         return(status);
       (void) WriteBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(MagickTrue);
     }
   if (LocaleCompare(image_info->magick,"iptc") == 0)
@@ -2241,7 +2238,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       unsigned char
         *info;
 
-      profile=GetImageProfile(image,"iptc");
+      profile=GetImageProfile(image,"8bim");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -2251,7 +2248,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       if (length == 0)
         ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
       (void) WriteBlob(image,length,info);
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(MagickTrue);
     }
   if (LocaleCompare(image_info->magick,"8BIMTEXT") == 0)
@@ -2265,7 +2262,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       AttachBlob(buff->blob,GetStringInfoDatum(profile),
@@ -2273,7 +2270,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       format8BIM(buff,image);
       (void) DetachBlob(buff->blob);
       buff=DestroyImage(buff);
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(MagickTrue);
     }
   if (LocaleCompare(image_info->magick,"8BIMWTEXT") == 0)
@@ -2286,7 +2283,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       unsigned char
         *info;
 
-      profile=GetImageProfile(image,"iptc");
+      profile=GetImageProfile(image,"8bim");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       info=GetStringInfoDatum(profile);
@@ -2297,14 +2294,14 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
-      buff=AllocateImage((ImageInfo *) NULL);
+      buff=AcquireImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       AttachBlob(buff->blob,info,length);
       formatIPTC(buff,image);
       (void) DetachBlob(buff->blob);
       buff=DestroyImage(buff);
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(MagickTrue);
     }
   if (LocaleCompare(image_info->magick,"IPTCWTEXT") == 0)
@@ -2324,7 +2321,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
         return(status);
       (void) WriteBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(MagickTrue);
     }
   if ((LocaleCompare(image_info->magick,"ICC") == 0) ||
@@ -2341,7 +2338,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
         return(status);
       (void) WriteBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(MagickTrue);
     }
   return(MagickFalse);
