@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -478,7 +478,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     *p++=(char) c;
     if ((strchr("\n\r%",c) == (char *) NULL) &&
-        ((size_t) (p-command) < MaxTextExtent))
+        ((size_t) (p-command) < (MaxTextExtent-1)))
       continue;
     *p='\0';
     p=command;
@@ -557,7 +557,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           GetStringInfoDatum(profile)[i]=(unsigned char) c;
           *p++=(char) c;
           if ((strchr("\n\r%",c) == (char *) NULL) &&
-              ((size_t) (p-command) < MaxTextExtent))
+              ((size_t) (p-command) < (MaxTextExtent-1)))
             continue;
           *p='\0';
           p=command;
@@ -656,10 +656,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) CopyMagickString(command,"/setpagedevice {pop} bind 1 index where {"
     "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n"
     "<</UseCIEColor true>>setpagedevice\n",MaxTextExtent);
-  (void) write(file,command,(unsigned int) strlen(command));
+  status=write(file,command,(unsigned int) strlen(command));
   (void) FormatMagickString(translate_geometry,MaxTextExtent,
     "%g %g translate\n",-bounds.x1,-bounds.y1);
-  (void) write(file,translate_geometry,strlen(translate_geometry));
+  status=write(file,translate_geometry,strlen(translate_geometry));
   file=close(file)-1;
   /*
     Render Postscript with the Ghostscript delegate.
@@ -965,7 +965,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       q=PopHexPixel(hex_digits,ScaleQuantumToChar(pixel.green),q); \
       q=PopHexPixel(hex_digits,ScaleQuantumToChar(pixel.blue),q); \
     } \
-  q=PopHexPixel(hex_digits,MagickMin(length,0xff),q); \
+  q=PopHexPixel(hex_digits,(const unsigned long) MagickMin(length,0xff),q); \
 }
 
   static const char
@@ -1398,7 +1398,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     scale.y=(double) (geometry.height*delta.y)/resolution.y;
     geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info,
+      &image->exception);
     if (image->gravity != UndefinedGravity)
       {
         geometry.x=(-page_info.x);
@@ -1953,7 +1954,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                       if (x > 0)
                         {
                           q=PopHexPixel(hex_digits,index,q);
-                          q=PopHexPixel(hex_digits,MagickMin(length,0xff),q);
+                          q=PopHexPixel(hex_digits,(unsigned long)
+                            MagickMin(length,0xff),q);
                           i++;
                           if ((q-pixels+6) >= 80)
                             {
@@ -1969,7 +1971,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 q=PopHexPixel(hex_digits,index,q);
-                q=PopHexPixel(hex_digits,MagickMin(length,0xff),q);
+                q=PopHexPixel(hex_digits,(unsigned long)
+                  MagickMin(length,0xff),q);
                 if (image->previous == (Image *) NULL)
                   {
                     status=SetImageProgress(image,SaveImageTag,y,image->rows);
