@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -180,7 +180,7 @@ static MagickBooleanType DecodeImage(Image *image,const long channel)
             }
             case 4:
             {
-              q->opacity=(Quantum) pixel;
+              q->opacity=(Quantum) (QuantumRange-pixel);
               break;
             }
             default:
@@ -446,18 +446,12 @@ static const char *ModeToString( PSDImageType inType )
   }
 }
 
-static MagickBooleanType ParseImageResourceBlocks(const ImageInfo *image_info,
-  Image *image,const unsigned char *blocks,size_t length)
+static MagickBooleanType ParseImageResourceBlocks(Image *image,
+  const unsigned char *blocks,size_t length)
 {
   const unsigned char
     *p;
 
-  QuantumInfo
-    quantum_info;
-
-  QuantumState
-    quantum_state;
-
   StringInfo
     *profile;
 
@@ -471,8 +465,6 @@ static MagickBooleanType ParseImageResourceBlocks(const ImageInfo *image_info,
 
   if (length < 16)
     return(MagickFalse);
-  GetQuantumInfo(image_info,&quantum_info);
-  InitializeQuantumState(&quantum_info,LSBEndian,&quantum_state);
   profile=AcquireStringInfo(length);
   SetStringInfoDatum(profile,blocks);
   (void) SetImageProfile(image,"8bim",profile);
@@ -481,10 +473,10 @@ static MagickBooleanType ParseImageResourceBlocks(const ImageInfo *image_info,
   {
     if (LocaleNCompare((char *) p,"8BIM",4) != 0)
       break;
-    p=PushLongPixel(quantum_state.endian,p,&long_sans);
-    p=PushShortPixel(quantum_state.endian,p,&id);
-    p=PushShortPixel(quantum_state.endian,p,&short_sans);
-    p=PushLongPixel(quantum_state.endian,p,&count);
+    p=PushLongPixel(LSBEndian,p,&long_sans);
+    p=PushShortPixel(LSBEndian,p,&id);
+    p=PushShortPixel(LSBEndian,p,&short_sans);
+    p=PushLongPixel(LSBEndian,p,&count);
     switch (id)
     {
       case 0x03ed:
@@ -495,16 +487,16 @@ static MagickBooleanType ParseImageResourceBlocks(const ImageInfo *image_info,
         /*
           Resolution info.
         */
-        p=PushShortPixel(quantum_state.endian,p,&resolution);
+        p=PushShortPixel(LSBEndian,p,&resolution);
         image->x_resolution=(double) resolution;
-        p=PushShortPixel(quantum_state.endian,p,&short_sans);
-        p=PushShortPixel(quantum_state.endian,p,&short_sans);
-        p=PushShortPixel(quantum_state.endian,p,&short_sans);
-        p=PushShortPixel(quantum_state.endian,p,&resolution);
+        p=PushShortPixel(LSBEndian,p,&short_sans);
+        p=PushShortPixel(LSBEndian,p,&short_sans);
+        p=PushShortPixel(LSBEndian,p,&short_sans);
+        p=PushShortPixel(LSBEndian,p,&resolution);
         image->y_resolution=(double) resolution;
-        p=PushShortPixel(quantum_state.endian,p,&short_sans);
-        p=PushShortPixel(quantum_state.endian,p,&short_sans);
-        p=PushShortPixel(quantum_state.endian,p,&short_sans);
+        p=PushShortPixel(LSBEndian,p,&short_sans);
+        p=PushShortPixel(LSBEndian,p,&short_sans);
+        p=PushShortPixel(LSBEndian,p,&short_sans);
         break;
       }
       default:
@@ -772,7 +764,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           blocks=(unsigned char *) RelinquishMagickMemory(blocks);
           ThrowReaderException(CorruptImageError,"ImproperImageHeader");
         }
-      (void) ParseImageResourceBlocks(image_info,image,blocks,length);
+      (void) ParseImageResourceBlocks(image,blocks,length);
       blocks=(unsigned char *) RelinquishMagickMemory(blocks);
     }
   /*
@@ -783,11 +775,6 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   /*
     Layer and mask block.
   */
@@ -1509,7 +1496,7 @@ static void WriteOneChannel(const ImageInfo *image_info,Image *image,
     y;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   register const PixelPacket
     *p;
@@ -1519,17 +1506,18 @@ static void WriteOneChannel(const ImageInfo *image_info,Image *image,
 
   if (tmp_image->depth > 8)
     tmp_image->depth=16;
-  GetQuantumInfo(image_info,&quantum_info);
+  quantum_info=AcquireQuantumInfo(image_info,image);
   packet_size=tmp_image->depth > 8UL ? 2UL : 1UL;
   for (y=0; y < (long) tmp_image->rows; y++)
   {
     p=GetVirtualPixels(tmp_image,0,y,tmp_image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
     break;
-    (void) ExportQuantumPixels(tmp_image,(ViewInfo *) NULL,&quantum_info,
+    (void) ExportQuantumPixels(tmp_image,(ViewInfo *) NULL,quantum_info,
       quantum_type,pixels,&image->exception);
     (void) WriteBlob(image,packet_size*tmp_image->columns,pixels);
   }
+  quantum_info=DestroyQuantumInfo(quantum_info);
 }
 
 static MagickBooleanType WriteImageChannels(const ImageInfo *image_info,
