@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -98,7 +98,7 @@ typedef struct _ExifInfo
 
 static const ExifInfo
   exif_info[] = {
-    { EXIFTAG_EXPOSURETIME, TIFF_RATIONAL, "exif:ExposureTime" }, 
+    { EXIFTAG_EXPOSURETIME, TIFF_RATIONAL, "exif:ExposureTime" },
     { EXIFTAG_FNUMBER, TIFF_RATIONAL, "exif:FNumber" }, 
     { EXIFTAG_EXPOSUREPROGRAM, TIFF_SHORT, "exif:ExposureProgram" }, 
     { EXIFTAG_SPECTRALSENSITIVITY, TIFF_ASCII, "exif:SpectralSensitivity" }, 
@@ -333,16 +333,18 @@ static void TIFFGetProfiles(TIFF *tiff,Image *image)
   unsigned char
     *profile;
 
-  length=0;
 #if defined(TIFFTAG_ICCPROFILE)
+  length=0;
   if (TIFFGetField(tiff,TIFFTAG_ICCPROFILE,&length,&profile) == 1)
     (void) ReadProfile(image,"icc",profile,(long) length);
 #endif
 #if defined(TIFFTAG_PHOTOSHOP)
+  length=0;
   if (TIFFGetField(tiff,TIFFTAG_PHOTOSHOP,&length,&profile) == 1)
     (void) ReadProfile(image,"8bim",profile,(long) length);
 #endif
 #if defined(TIFFTAG_RICHTIFFIPTC)
+  length=0;
   if (TIFFGetField(tiff,TIFFTAG_RICHTIFFIPTC,&length,&profile) == 1)
     {
       if (TIFFIsByteSwapped(tiff) != 0)
@@ -351,9 +353,11 @@ static void TIFFGetProfiles(TIFF *tiff,Image *image)
     }
 #endif
 #if defined(TIFFTAG_XMLPACKET)
+  length=0;
   if (TIFFGetField(tiff,TIFFTAG_XMLPACKET,&length,&profile) == 1)
     (void) ReadProfile(image,"xmp",profile,(long) length);
 #endif
+  length=0;
   if (TIFFGetField(tiff,37724,&length,&profile) == 1)
     (void) ReadProfile(image,"tiff:37724",profile,(long) length);
 }
@@ -626,7 +630,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     *pixels;
 
   unsigned long
-    lsb_first;
+    lsb_first,
+    pad;
 
   /*
     Open image.
@@ -678,7 +683,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     }
   do
   {
-    if (image_info->verbose != MagickFalse)
+    if (0 && (image_info->verbose != MagickFalse))
       TIFFPrintDirectory(tiff,stdout,MagickFalse);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_COMPRESSION,&compress_tag);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_ORIENTATION,&orientation);
@@ -847,12 +852,16 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Image depth: %lu",
         image->depth);
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if (sample_format == SAMPLEFORMAT_UINT)
-      (void) SetQuantumFormat(image,quantum_info,UnsignedQuantumFormat);
+      status=SetQuantumFormat(image,quantum_info,UnsignedQuantumFormat);
     if (sample_format == SAMPLEFORMAT_INT)
-      (void) SetQuantumFormat(image,quantum_info,SignedQuantumFormat);
+      status=SetQuantumFormat(image,quantum_info,SignedQuantumFormat);
     if (sample_format == SAMPLEFORMAT_IEEEFP)
-      (void) SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
+      status=SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
+    if (status == MagickFalse)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     status=MagickTrue;
     switch (photometric)
     {
@@ -924,13 +933,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     value=(unsigned short) image->scene;
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_PAGENUMBER,&value,&pages);
     image->scene=value;
-    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
-      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
-        break;
-    if (SetImageExtent(image,0,0) == MagickFalse)
+    if (image_info->ping != MagickFalse)
       {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
+        if (image_info->number_scenes != 0)
+          if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+            break;
+        goto next_tiff_frame;
       }
     method=ReadGenericMethod;
     if (TIFFGetField(tiff,TIFFTAG_ROWSPERSTRIP,&rows_per_strip) != 0)
@@ -1002,30 +1010,31 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             }
           }
         quantum_type=IndexQuantum;
-        quantum_info->pad=(size_t) MagickMax((size_t) samples_per_pixel-1,0);
+        pad=(size_t) MagickMax((size_t) samples_per_pixel-1,0);
         if (image->matte != MagickFalse)
           {
             if (image->storage_class != PseudoClass)
               {
                 quantum_type=samples_per_pixel == 1 ? AlphaQuantum :
                   GrayAlphaQuantum;
-                quantum_info->pad=(size_t) MagickMax((size_t) samples_per_pixel-
-                  2,0);
+                pad=(size_t) MagickMax((size_t) samples_per_pixel-2,0);
               }
             else
               {
                 quantum_type=IndexAlphaQuantum;
-                quantum_info->pad=(size_t) MagickMax((size_t) samples_per_pixel-
-                  2,0);
+                pad=(size_t) MagickMax((size_t) samples_per_pixel-2,0);
               }
           }
         else
           if (image->storage_class != PseudoClass)
             {
               quantum_type=GrayQuantum;
-              quantum_info->pad=(size_t) MagickMax((size_t) samples_per_pixel-
-                1,0);
+              pad=(size_t) MagickMax((size_t) samples_per_pixel-1,0);
             }
+        status=SetQuantumPad(image,quantum_info,pad*((bits_per_sample+7) >> 3));
+        if (status == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        pixels=GetQuantumPixels(quantum_info);
         for (y=0; y < (long) image->rows; y++)
         {
           int
@@ -1037,9 +1046,6 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          /*
-            Transfer image scanline.
-          */
           length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
             quantum_type,pixels,exception);
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
@@ -1055,25 +1061,9 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       }
       case ReadRGBAMethod:
       {
-        long
-          row;
-
-        QuantumInfo
-          **quantum_info;
-
-        size_t
-          extent;
-
-        unsigned long
-          pad;
-
-        ViewInfo
-          **image_view;
-
         /*
           Convert TIFF image to DirectClass MIFF image.
         */
-        row=0;
         pad=(size_t) MagickMax((size_t) samples_per_pixel-3,0);
         quantum_type=RGBQuantum;
         if (image->matte != MagickFalse)
@@ -1091,82 +1081,32 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
                 pad=(size_t) MagickMax((size_t) samples_per_pixel-5,0);
               }
           }
-        quantum_info=AcquireQuantumInfoThreadSet(image_info,image);
-        if (quantum_info == (QuantumInfo **) NULL)
+        status=SetQuantumPad(image,quantum_info,pad*((bits_per_sample+7) >> 3));
+        if (status == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
-          (void) SetQuantumPad(image,quantum_info[i],pad);
-        extent=GetQuantumExtent(image,quantum_info[0],quantum_type);
-        image_view=AcquireCacheViewThreadSet(image);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,1) shared(status)
-#endif
+        pixels=GetQuantumPixels(quantum_info);
         for (y=0; y < (long) image->rows; y++)
         {
-          long
-            offset;
+          int
+            status;
 
-          MagickBooleanType
-            sync;
-
-          register long
-            id;
-
-          register PixelPacket
-            *q;
-
-          size_t
-            length;
-
-          unsigned char
-            *pixels;
-
-          if (status == MagickFalse)
-            continue;
-          id=GetCacheViewThreadId();
-          pixels=GetQuantumPixels(quantum_info[id]);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
-#endif
-          {
-            int
-              tiff_status;
-
-            tiff_status=TIFFReadPixels(tiff,bits_per_sample,0,row,(char *)
-              pixels);
-            if (tiff_status == -1)
-              status=MagickFalse;
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (image->previous == (Image *) NULL))
-              {
-                MagickBooleanType
-                  proceed;
-
-                proceed=SetImageProgress(image,LoadImageTag,row,image->rows);
-                if (proceed == MagickFalse)
-                  status=MagickFalse;
-              }
-            offset=row++;
-          }
-          q=QueueCacheViewAuthenticPixels(image_view[id],0,offset,
-            image->columns,1,exception);
+          status=TIFFReadPixels(tiff,bits_per_sample,0,y,(char *) pixels);
+          if (status == -1)
+            break;
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
-            {
-              status=MagickFalse;
-              continue;
-            }
-          length=ImportQuantumPixels(image,image_view[id],quantum_info[id],
+            break;
+          length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
             quantum_type,pixels,exception);
-          if (length != extent)
-            status=MagickFalse;
-          sync=SyncCacheViewAuthenticPixels(image_view[id],exception);
-          if (sync == MagickFalse)
-            status=MagickFalse;
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
+            break;
+          if (image->previous == (Image *) NULL)
+            {
+              status=SetImageProgress(image,LoadImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
-        image_view=DestroyCacheViewThreadSet(image_view);
-        quantum_info=DestroyQuantumInfoThreadSet(quantum_info);
-        if (status == MagickFalse)
-          ThrowReaderException(CorruptImageError,"UnableToReadImageData");
         break;
       }
       case ReadCMYKAMethod:
@@ -1425,6 +1365,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       }
     }
     SetQuantumImageType(image,quantum_type);
+  next_tiff_frame:
     if ((photometric == PHOTOMETRIC_LOGL) ||
         (photometric == PHOTOMETRIC_MINISBLACK) ||
         (photometric == PHOTOMETRIC_MINISWHITE))
@@ -1525,7 +1466,7 @@ ModuleExport unsigned long RegisterTIFFImage(void)
       i;
 
     p=TIFFGetVersion();
-    for (i=0; (i < MaxTextExtent-1) && (*p != 0) && (*p != '\n'); i++)
+    for (i=0; (i < (MaxTextExtent-1)) && (*p != 0) && (*p != '\n'); i++)
       version[i]=(*p++);
     version[i]='\0';
   }
@@ -1675,8 +1616,6 @@ static MagickBooleanType WritePTIFImage(const ImageInfo *image_info,
       rows/=2;
       pyramid_image=ResizeImage(next,columns,rows,UndefinedFilter,image->blur,
         &image->exception);
-      if (pyramid_image == (Image *) NULL)
-        break;
       AppendImageToList(&images,pyramid_image);
     }
   }
@@ -1698,7 +1637,6 @@ static MagickBooleanType WritePTIFImage(const ImageInfo *image_info,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%                                                                             %
 %   W r i t e T I F F I m a g e                                               %
 %                                                                             %
 %                                                                             %
@@ -1815,34 +1753,35 @@ static int32 TIFFWritePixels(TIFF *tiff,TIFFInfo *tiff_info,long row,
     Write tile to TIFF image.
   */
   status=0;
-  bytes_per_pixel=TIFFTileSize(tiff)/(long)
-    (tiff_info->tile_geometry.height*tiff_info->tile_geometry.width);
-  number_tiles=(image->columns+tiff_info->tile_geometry.width/2)/
+  bytes_per_pixel=TIFFTileSize(tiff)/(long) (tiff_info->tile_geometry.height*
+    tiff_info->tile_geometry.width);
+  number_tiles=(image->columns+tiff_info->tile_geometry.width)/
     tiff_info->tile_geometry.width;
   for (i=0; i < (long) number_tiles; i++)
   {
-    tile_width=(i == (long) (number_tiles-1)) ?
-      image->columns-(i*tiff_info->tile_geometry.width) : tiff_info->tile_geometry.width;
+    tile_width=(i == (long) (number_tiles-1)) ? image->columns-(i*
+      tiff_info->tile_geometry.width) : tiff_info->tile_geometry.width;
     for (j=0; j < (long) ((row % tiff_info->tile_geometry.height)+1); j++)
       for (k=0; k < (long) tile_width; k++)
       {
         if (bytes_per_pixel == 0)
           {
-             p=tiff_info->scanlines+(j*TIFFScanlineSize(tiff)+
-               (i*tiff_info->tile_geometry.width+k)/8);
-             q=tiff_info->pixels+(j*TIFFTileRowSize(tiff)+k/8);
-             *q++=(*p++);
-             continue;
+            p=tiff_info->scanlines+(j*TIFFScanlineSize(tiff)+(i*
+              tiff_info->tile_geometry.width+k)/8);
+            q=tiff_info->pixels+(j*TIFFTileRowSize(tiff)+k/8);
+            *q++=(*p++);
+            continue;
           }
-        p=tiff_info->scanlines+(j*TIFFScanlineSize(tiff)+
-          (i*tiff_info->tile_geometry.width+k)*bytes_per_pixel);
+        p=tiff_info->scanlines+(j*TIFFScanlineSize(tiff)+(i*
+          tiff_info->tile_geometry.width+k)*bytes_per_pixel);
         q=tiff_info->pixels+(j*TIFFTileRowSize(tiff)+k*bytes_per_pixel);
         for (l=0; l < bytes_per_pixel; l++)
           *q++=(*p++);
       }
     status=TIFFWriteTile(tiff,tiff_info->pixels,(uint32) (i*
-      tiff_info->tile_geometry.width),(uint32) ((row/tiff_info->tile_geometry.height)*
-      tiff_info->tile_geometry.height),0,sample);
+      tiff_info->tile_geometry.width),(uint32) ((row/
+      tiff_info->tile_geometry.height)*tiff_info->tile_geometry.height),0,
+      sample);
     if (status < 0)
       break;
   }
@@ -1892,18 +1831,11 @@ static void TIFFSetProfiles(TIFF *tiff,Image *image)
           GetStringInfoLength(iptc_profile)/4,GetStringInfoDatum(iptc_profile));
         iptc_profile=DestroyStringInfo(iptc_profile);
       }
-    if (LocaleCompare(name,"8bim") == 0)
-      {
 #if defined(TIFFTAG_PHOTOSHOP)
-        uint32
-          length;
-
-        length=(uint32) (GetStringInfoLength(profile)+(GetStringInfoLength(
-          profile) & 0x01));
-        (void) TIFFSetField(tiff,TIFFTAG_PHOTOSHOP,length,GetStringInfoDatum(
-          profile));
+    if (LocaleCompare(name,"8bim") == 0)
+      (void) TIFFSetField(tiff,TIFFTAG_PHOTOSHOP,(uint32)
+        GetStringInfoLength(profile),GetStringInfoDatum(profile));
 #endif
-      }
     if (LocaleCompare(name,"tiff:37724") == 0)
       (void) TIFFSetField(tiff,37724,(uint32)GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
@@ -2127,17 +2059,28 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     /*
       Initialize TIFF fields.
     */
+    if ((image_info->type != UndefinedType) &&
+        (image_info->type != OptimizeType))
+      (void) SetImageType(image,image_info->type);
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     if ((image->storage_class != PseudoClass) &&
         (quantum_info->format == UndefinedQuantumFormat) &&
         (IsHighDynamicRangeImage(image,&image->exception) != MagickFalse))
-      (void) SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
+      {
+        status=SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
+        if (status == MagickFalse)
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     if ((LocaleCompare(image_info->magick,"PTIF") == 0) &&
         (GetPreviousImageInList(image) != (Image *) NULL))
       (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_REDUCEDIMAGE);
     (void) TIFFSetField(tiff,TIFFTAG_IMAGELENGTH,(uint32) image->rows);
     (void) TIFFSetField(tiff,TIFFTAG_IMAGEWIDTH,(uint32) image->columns);
-    compression=image_info->compression;
+    compression=image->compression;
+    if (image_info->compression != UndefinedCompression)
+      compression=image_info->compression;
     switch (compression)
     {
       case FaxCompression:
@@ -2290,7 +2233,12 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,1);
                 if ((image_info->depth == 0) &&
                     (IsMonochromeImage(image,&image->exception) != MagickFalse))
-                  (void) SetQuantumDepth(image,quantum_info,1);
+                  {
+                    status=SetQuantumDepth(image,quantum_info,1);
+                    if (status == MagickFalse)
+                      ThrowWriterException(ResourceLimitError,
+                        "MemoryAllocationFailed");
+                  }
               }
             else
               if (image->storage_class == PseudoClass)
@@ -2303,7 +2251,10 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                   depth=1;
                   while ((GetQuantumRange(depth)+1) < image->colors)
                     depth<<=1;
-                  (void) SetQuantumDepth(image,quantum_info,depth);
+                  status=SetQuantumDepth(image,quantum_info,depth);
+                  if (status == MagickFalse)
+                    ThrowWriterException(ResourceLimitError,
+                      "MemoryAllocationFailed");
                 }
           }
       }
@@ -2406,8 +2357,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         MagickStatusType
           flags;
 
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,
-          rows_per_strip+(16-(rows_per_strip % 16)));
+        rows_per_strip+=(16-(rows_per_strip % 16));
         if (image->quality != 0)
           (void) TIFFSetField(tiff,TIFFTAG_JPEGQUALITY,image->quality);
         if (image_info->quality != UndefinedCompressionQuality)
@@ -2443,7 +2393,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
       case COMPRESSION_ADOBE_DEFLATE:
       {
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,image->rows);
+        rows_per_strip=image->rows;
         (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_BITSPERSAMPLE,
           &bits_per_sample);
         if (((photometric == PHOTOMETRIC_RGB) ||
@@ -2460,18 +2410,17 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         /*
           Byte-aligned EOL.
         */
+        rows_per_strip=image->rows;
         (void) TIFFSetField(tiff,TIFFTAG_GROUP3OPTIONS,4);
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,image->rows);
         break;
       }
       case COMPRESSION_CCITTFAX4:
       {
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,image->rows);
+        rows_per_strip=image->rows;
         break;
       }
       case COMPRESSION_LZW:
       {
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,rows_per_strip);
         (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_BITSPERSAMPLE,
           &bits_per_sample);
         if (((photometric == PHOTOMETRIC_RGB) ||
@@ -2481,11 +2430,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         break;
       }
       default:
-      {
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,rows_per_strip);
         break;
-      }
     }
+    (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,rows_per_strip);
     if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
       {
         unsigned short
@@ -2541,10 +2488,21 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
     if (image->orientation != UndefinedOrientation)
       (void) TIFFSetField(tiff,TIFFTAG_ORIENTATION,(uint16) image->orientation);
+    (void) TIFFSetProfiles(tiff,image);
+    {
+      uint16
+        page,
+        pages;
+
+      page=(uint16) scene;
+      pages=GetImageListLength(image);
+      if ((image_info->adjoin != MagickFalse) && (pages > 1))
+        (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_PAGE);
+      (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,page,pages);
+    }
     (void) TIFFSetProperties(tiff,image);
     if (0)
       (void) TIFFSetEXIFProperties(tiff,image);
-    (void) TIFFSetProfiles(tiff,image);
     /*
       Write image scanlines.
     */
@@ -2766,7 +2724,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     }
     quantum_info=DestroyQuantumInfo(quantum_info);
     DestroyTIFFInfo(&tiff_info);
-    if (image_info->verbose == MagickTrue)
+    if (0 && (image_info->verbose == MagickTrue))
       TIFFPrintDirectory(tiff,stdout,MagickFalse);
     (void) TIFFWriteDirectory(tiff);
     image->endian=MSBEndian;
