@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -216,7 +216,8 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   SetGeometryInfo(&geometry_info);
   if (annotate_info->geometry != (char *) NULL)
     {
-      (void) ParsePageGeometry(image,annotate_info->geometry,&geometry);
+      (void) ParsePageGeometry(image,annotate_info->geometry,&geometry,
+        &image->exception);
       (void) ParseGeometry(annotate_info->geometry,&geometry_info);
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
@@ -1490,16 +1491,22 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     glyph.id=FT_Get_Char_Index(face,text[i]);
     if (glyph.id == 0)
       glyph.id=FT_Get_Char_Index(face,'?');
-    if ((glyph.id != 0) && (last_glyph.id != 0) && FT_HAS_KERNING(face))
+    if ((glyph.id != 0) && (last_glyph.id != 0))
       {
-        FT_Vector
-          kerning;
+        if (draw_info->kerning != 0.0)
+          origin.x+=64.0*draw_info->kerning;
+        else
+          if (FT_HAS_KERNING(face))
+            {
+              FT_Vector
+                kerning;
 
-        status=FT_Get_Kerning(face,last_glyph.id,glyph.id,ft_kerning_default,
-          &kerning);
-        if (status == 0)
-          origin.x+=kerning.x;
-      }
+              status=FT_Get_Kerning(face,last_glyph.id,glyph.id,
+                ft_kerning_default,&kerning);
+              if (status == 0)
+                origin.x+=kerning.x;
+            }
+        }
     glyph.origin=origin;
     status=FT_Load_Glyph(face,glyph.id,flags);
     if (status != 0)
@@ -1549,16 +1556,16 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
           status;
 
         ViewInfo
-          **image_view;
+          *image_view;
 
         /*
           Rasterize the glyph.
         */
         status=MagickTrue;
         exception=(&image->exception);
-        image_view=AcquireCacheViewThreadSet(image);
+        image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
         for (y=0; y < (long) bitmap->bitmap.rows; y++)
         {
@@ -1577,7 +1584,6 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             fill_color;
 
           register long
-            id,
             x;
 
           register PixelPacket
@@ -1592,8 +1598,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
           y_offset=(long) (point.y+y+0.5);
           if ((y_offset < 0) || (y_offset >= (long) image->rows))
             continue;
-          id=GetCacheViewThreadId();
-          q=GetCacheViewAuthenticPixels(image_view[id],x_offset,y_offset,
+          q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,
             bitmap->bitmap.width,1,exception);
           active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
           p=bitmap->bitmap.buffer+y*bitmap->bitmap.width;
@@ -1611,8 +1616,8 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             if (draw_info->text_antialias == MagickFalse)
               fill_opacity=fill_opacity > 0.5 ? 1.0 : 0.0;
             if (active == MagickFalse)
-              q=GetCacheViewAuthenticPixels(image_view[id],x_offset,y_offset,1,
-                1,exception);
+              q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,1,1,
+                exception);
             if (q == (PixelPacket *) NULL)
               {
                 p++;
@@ -1625,22 +1630,26 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
             if (active == MagickFalse)
               {
-                sync=SyncCacheViewAuthenticPixels(image_view[id],exception);
+                sync=SyncCacheViewAuthenticPixels(image_view,exception);
                 if (sync == MagickFalse)
                   status=MagickFalse;
               }
             p++;
             q++;
           }
-          sync=SyncCacheViewAuthenticPixels(image_view[id],exception);
+          sync=SyncCacheViewAuthenticPixels(image_view,exception);
           if (sync == MagickFalse)
             status=MagickFalse;
         }
-        image_view=DestroyCacheViewThreadSet(image_view);
+        image_view=DestroyCacheView(image_view);
       }
     if ((bitmap->left+bitmap->bitmap.width) > metrics->width)
       metrics->width=bitmap->left+bitmap->bitmap.width;
-    origin.x+=face->glyph->advance.x;
+    if ((draw_info->interword_spacing != 0.0) && (isspace(text[i]) != 0) &&
+        (i != 0) && (isspace(text[i-1]) == 0))
+      origin.x+=64.0*draw_info->interword_spacing;
+    else
+      origin.x+=face->glyph->advance.x;
     metrics->origin.x=origin.x;
     metrics->origin.y=origin.y;
     if (last_glyph.id != 0)
