@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -37,71 +37,72 @@ typedef void
   *Cache;
 
 typedef const IndexPacket
-  *(*AcquireIndexesFromHandler)(const Image *);
+  *(*GetVirtualIndexesFromHandler)(const Image *);
 
 typedef IndexPacket
-  *(*GetIndexesFromHandler)(const Image *);
+  *(*GetAuthenticIndexesFromHandler)(const Image *);
 
 typedef MagickBooleanType
+  (*GetOneAuthenticPixelFromHandler)(Image *,const long,const long,
+    PixelPacket *,ExceptionInfo *),
   (*GetOneVirtualPixelFromHandler)(const Image *,const VirtualPixelMethod,
     const long,const long,PixelPacket *,ExceptionInfo *),
-  (*GetOneAuthenticPixelFromHandler)(Image *,const long,const long,PixelPacket *,
-    ExceptionInfo *),
-  (*SyncPixelHandler)(Image *,ExceptionInfo *);
+  (*SyncAuthenticPixelsHandler)(Image *,ExceptionInfo *);
 
 typedef const PixelPacket
-  *(*AcquirePixelHandler)(const Image *,const VirtualPixelMethod,const long,
+  *(*GetVirtualPixelHandler)(const Image *,const VirtualPixelMethod,const long,
     const long,const unsigned long,const unsigned long,ExceptionInfo *),
-  *(*AcquirePixelsFromHandler)(const Image *);
+  *(*GetVirtualPixelsHandler)(const Image *);
 
 typedef PixelPacket
-  *(*GetPixelHandler)(Image *,const long,const long,const unsigned long,
-    const unsigned long,ExceptionInfo *);
+  *(*GetAuthenticPixelsHandler)(Image *,const long,const long,
+    const unsigned long,const unsigned long,ExceptionInfo *);
 
 typedef PixelPacket
-  *(*GetPixelsFromHandler)(const Image *);
+  *(*GetAuthenticPixelsFromHandler)(const Image *);
 
 typedef PixelPacket
-  *(*SetPixelHandler)(Image *,const long,const long,const unsigned long,
-    const unsigned long,ExceptionInfo *);
+  *(*QueueAuthenticPixelsHandler)(Image *,const long,const long,
+    const unsigned long,const unsigned long,ExceptionInfo *);
 
 typedef void
   (*DestroyPixelHandler)(Image *);
 
 typedef struct _CacheMethods
 {
-  AcquirePixelsFromHandler
-    acquire_pixels_from_handler;
+  GetVirtualPixelHandler
+    get_virtual_pixel_handler;
 
-  AcquireIndexesFromHandler
-    acquire_indexes_from_handler;
+  GetVirtualPixelsHandler
+    get_virtual_pixels_handler;
+
+  GetVirtualIndexesFromHandler
+    get_virtual_indexes_from_handler;
 
   GetOneVirtualPixelFromHandler
-    acquire_one_pixel_from_handler;
+    get_one_virtual_pixel_from_handler;
 
-  AcquirePixelHandler
-    acquire_pixel_handler;
+  GetAuthenticPixelsHandler
+    get_authentic_pixels_handler;
+
+  GetAuthenticIndexesFromHandler
+    get_authentic_indexes_from_handler;
+
+  GetOneAuthenticPixelFromHandler
+    get_one_authentic_pixel_from_handler;
+
+  GetAuthenticPixelsFromHandler
+    get_authentic_pixels_from_handler;
+
+  QueueAuthenticPixelsHandler
+    queue_authentic_pixels_handler;
+
+  SyncAuthenticPixelsHandler
+    sync_authentic_pixels_handler;
 
   DestroyPixelHandler
     destroy_pixel_handler;
 
-  GetIndexesFromHandler
-    get_indexes_from_handler;
-
-  GetOneAuthenticPixelFromHandler
-    get_one_pixel_from_handler;
-
-  GetPixelHandler
-    get_pixel_handler;
-
-  GetPixelsFromHandler
-    get_pixels_from_handler;
-
-  SetPixelHandler
-    set_pixel_handler;
-
-  SyncPixelHandler
-    sync_pixel_handler;
 } CacheMethods;
 
 typedef struct _NexusInfo
@@ -109,19 +110,13 @@ typedef struct _NexusInfo
 
 typedef struct _CacheInfo
 {
-  NexusInfo
-    *nexus_info;
-
-  unsigned long
-    number_views;
-
   ClassType
     storage_class;
 
   ColorspaceType
     colorspace;
 
-  volatile CacheType
+  CacheType
     type;
 
   MagickBooleanType
@@ -137,24 +132,27 @@ typedef struct _CacheInfo
   MagickSizeType
     length;
 
+  VirtualPixelMethod
+    virtual_pixel_method;
+
+  unsigned long
+    number_threads;
+
+  NexusInfo
+    **nexus_info;
+
   PixelPacket
     *pixels;
 
   IndexPacket
     *indexes;
 
-  VirtualPixelMethod
-    virtual_pixel_method;
-
-  PixelPacket
-    virtual_pixel;
+  MagickBooleanType
+    active_index_channel;
 
   int
     file;
 
-  MagickSizeType
-    serial_number;
-
   char
     filename[MaxTextExtent],
     cache_filename[MaxTextExtent];
@@ -170,58 +168,69 @@ typedef struct _CacheInfo
 #elif defined(__WINDOWS__)
   DWORD
 #else
-  unsigned long
+  pid_t
 #endif
     id;
 
-  volatile long
+  long
     reference_count;
 
   SemaphoreInfo
     *semaphore,
-    *nexus_semaphore,
     *disk_semaphore;
 
+  time_t
+    timestamp;
+
   unsigned long
     signature;
 } CacheInfo;
 
 extern MagickExport Cache
-  DestroyCacheInfo(Cache),
-  ReferenceCache(Cache);
+  AcquirePixelCacheInfo(const unsigned long),
+  DestroyPixelCacheInfo(Cache),
+  GetImagePixelCache(Image *,const MagickBooleanType,ExceptionInfo *),
+  ReferencePixelCache(Cache);
 
 extern MagickExport ClassType
-  GetCacheClass(const Cache);
+  GetPixelCacheStorageClass(const Cache);
 
 extern MagickExport ColorspaceType
-  GetCacheColorspace(const Cache);
+  GetPixelCacheColorspace(const Cache);
 
 extern MagickExport const IndexPacket
-  *AcquireNexusIndexes(const Cache,const unsigned long);
+  *GetVirtualIndexesFromNexus(const Cache,NexusInfo *);
 
 extern MagickExport const PixelPacket
-  *AcquireNexusPixels(const Cache,const unsigned long);
+  *GetVirtualPixelsFromNexus(const Image *,const VirtualPixelMethod,const long,
+    const long,const unsigned long,const unsigned long,NexusInfo *,
+    ExceptionInfo *),
+  *GetVirtualPixelsNexus(const Cache,NexusInfo *);
 
 extern MagickExport IndexPacket
-  *GetNexusIndexes(const Cache,const unsigned long);
+  *GetPixelCacheNexusIndexes(const Cache,NexusInfo *);
 
 extern MagickExport MagickBooleanType
-  GetCacheInfo(Cache *);
+  SyncAuthenticPixelCacheNexus(Image *,NexusInfo *,ExceptionInfo *);
 
 extern MagickExport MagickSizeType
-  GetNexusExtent(const Cache,const unsigned long);
+  GetPixelCacheNexusExtent(const Cache,NexusInfo *);
+
+extern MagickExport NexusInfo
+  **AcquirePixelCacheNexus(const unsigned long),
+  **DestroyPixelCacheNexus(NexusInfo **,const unsigned long);
 
 extern MagickExport PixelPacket
-  *GetNexusPixels(const Cache,const unsigned long);
-
-extern MagickExport unsigned long
-  GetNexus(Cache);
+  *GetAuthenticPixelCacheNexus(Image *,const long,const long,const unsigned long,
+    const unsigned long,NexusInfo *,ExceptionInfo *),
+  *GetPixelCacheNexusPixels(const Cache,NexusInfo *),
+  *QueueAuthenticNexus(Image *,const long,const long,const unsigned long,
+    const unsigned long,NexusInfo *,ExceptionInfo *);
 
 extern MagickExport void
-  CloneCacheMethods(Cache,const Cache),
-  DestroyCacheNexus(Cache,const unsigned long),
-  GetCacheMethods(CacheMethods *),
-  SetCacheMethods(Cache,CacheMethods *);
+  ClonePixelCacheMethods(Cache,const Cache),
+  GetPixelCacheMethods(CacheMethods *),
+  SetPixelCacheMethods(Cache,CacheMethods *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
