@@ -16,7 +16,7 @@
 %                               March  2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -226,6 +226,8 @@ static AESInfo *AcquireAESInfo(void)
 %
 %      MagickBooleanType DecipherImage(Image *image,const char *passphrase,
 %        ExceptionInfo *exception)
+%      MagickBooleanType PasskeyDecipherImage(Image *image,
+%        const StringInfo *passkey,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -233,6 +235,8 @@ static AESInfo *AcquireAESInfo(void)
 %
 %    o passphrase: decipher cipher pixels with this passphrase.
 %
+%    o passkey: decrypt cipher pixels with this passkey.
+%
 %    o exception: return any errors or warnings in this structure.
 %
 */
@@ -246,6 +250,26 @@ static inline size_t MagickMin(const size_t x,const size_t y)
 
 MagickExport MagickBooleanType DecipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
+{
+  MagickBooleanType
+    status;
+
+  StringInfo
+    *passkey;
+
+  if (passphrase == (const char *) NULL)
+    return(MagickTrue);
+  passkey=StringToStringInfo(passphrase);
+  if (passkey == (StringInfo *) NULL)
+    return(MagickFalse);
+  status=PasskeyDecipherImage(image,passkey,exception);
+  passkey=DestroyStringInfo(passkey);
+  return(status);
+}
+
+
+MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
+  const StringInfo *passkey,ExceptionInfo *exception)
 {
 #define DecipherImageTag  "Decipher/Image "
 
@@ -294,6 +318,9 @@ MagickExport MagickBooleanType DecipherImage(Image *image,
     output_block[AESBlocksize],
     *pixels;
 
+  ViewInfo
+    *image_view;
+
   /*
     Generate decipher key and nonce.
   */
@@ -303,19 +330,20 @@ MagickExport MagickBooleanType DecipherImage(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (passphrase == (const char *) NULL)
+  if (passkey == (const StringInfo *) NULL)
     return(MagickTrue);
   quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   aes_info=AcquireAESInfo();
-  key=StringToStringInfo(passphrase);
+  key=CloneStringInfo(passkey);
   if (key == (StringInfo *) NULL)
     {
       aes_info=DestroyAESInfo(aes_info);
       quantum_info=DestroyQuantumInfo(quantum_info);
-      return(MagickFalse);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
     }
   nonce=SplitStringInfo(key,GetStringInfoLength(key)/2);
   if (nonce == (StringInfo *) NULL)
@@ -345,33 +373,35 @@ MagickExport MagickBooleanType DecipherImage(Image *image,
   */
   quantum_type=GetQuantumType(image,exception);
   pixels=GetQuantumPixels(quantum_info);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetAuthenticIndexQueue(image);
-    length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
-      quantum_type,pixels,exception);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
     p=pixels;
     for (x=0; x < (long) length; x++)
     {
       (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
-      EncipherAESBlock(aes_info,output_block,output_block); 
+      EncipherAESBlock(aes_info,output_block,output_block);
       (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
         sizeof(*input_block));
       input_block[AESBlocksize-1]=(*p);
       *p++^=(*output_block);
     }
-    (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
-      quantum_type,pixels,exception);
-    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+    (void) ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
     proceed=SetImageProgress(image,DecipherImageTag,y,image->rows);
     if (proceed == MagickFalse)
       break;
   }
+  image_view=DestroyCacheView(image_view);
   (void) DeleteImageProperty(image,"cipher:type");
   (void) DeleteImageProperty(image,"cipher:mode");
   (void) DeleteImageProperty(image,"cipher:nonce");
@@ -443,6 +473,8 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 %
 %      MagickBooleanType EncipherImage(Image *image,const char *passphrase,
 %        ExceptionInfo *exception)
+%      MagickBooleanType PasskeyEncipherImage(Image *image,
+%        const StringInfo *passkey,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -450,11 +482,33 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 %
 %    o passphrase: encipher pixels with this passphrase.
 %
+%    o passkey: decrypt cipher pixels with this passkey.
+%
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
 MagickExport MagickBooleanType EncipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
+{
+  MagickBooleanType
+    status;
+
+  StringInfo
+    *passkey;
+
+  if (passphrase == (const char *) NULL)
+    return(MagickTrue);
+  passkey=StringToStringInfo(passphrase);
+  if (passkey == (StringInfo *) NULL)
+    return(MagickFalse);
+  status=PasskeyEncipherImage(image,passkey,exception);
+  passkey=DestroyStringInfo(passkey);
+  return(status);
+}
+
+MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
+  const StringInfo *passkey,ExceptionInfo *exception)
 {
 #define EncipherImageTag  "Encipher/Image "
 
@@ -506,6 +560,9 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
     output_block[AESBlocksize],
     *pixels;
 
+  ViewInfo
+    *image_view;
+
   /*
     Generate encipher key and nonce.
   */
@@ -515,7 +572,7 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (passphrase == (const char *) NULL)
+  if (passkey == (const StringInfo *) NULL)
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
@@ -524,12 +581,13 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   aes_info=AcquireAESInfo();
-  key=StringToStringInfo(passphrase);
+  key=CloneStringInfo(passkey);
   if (key == (StringInfo *) NULL)
     {
       aes_info=DestroyAESInfo(aes_info);
       quantum_info=DestroyQuantumInfo(quantum_info);
-      return(MagickFalse);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
     }
   nonce=SplitStringInfo(key,GetStringInfoLength(key)/2);
   if (nonce == (StringInfo *) NULL)
@@ -564,33 +622,35 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
   */
   quantum_type=GetQuantumType(image,exception);
   pixels=GetQuantumPixels(quantum_info);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetAuthenticIndexQueue(image);
-    length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
-      quantum_type,pixels,exception);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
     p=pixels;
     for (x=0; x < (long) length; x++)
     {
       (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
-      EncipherAESBlock(aes_info,output_block,output_block); 
+      EncipherAESBlock(aes_info,output_block,output_block);
       *p^=(*output_block);
       (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
         sizeof(*input_block));
       input_block[AESBlocksize-1]=(*p++);
     }
-    (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
-      quantum_type,pixels,exception);
-    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+    (void) ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
     proceed=SetImageProgress(image,EncipherImageTag,y,image->rows);
     if (proceed == MagickFalse)
       break;
   }
+  image_view=DestroyCacheView(image_view);
   image->taint=MagickFalse;
   /*
     Free resources.
