@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -305,7 +305,7 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
         **image_list;
 
       int
-        number_image_list;
+        number_images;
 
       register long
         i;
@@ -317,11 +317,11 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
       if (command == (char *) NULL)
         return((Image *) NULL);
       StripString(command);
-      image_list=StringToArgv(command,&number_image_list);
+      image_list=StringToArgv(command,&number_images);
       command=DestroyString(command);
       read_info=CloneImageInfo(image_info);
       images=NewImageList();
-      for (i=1; i < number_image_list; i++)
+      for (i=1; i < number_images; i++)
       {
         (void) CopyMagickString(read_info->filename,image_list[i],
           MaxTextExtent);
@@ -331,7 +331,7 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
         AppendImageToList(&images,image);
       }
       read_info=DestroyImageInfo(read_info);
-      for (i=1; i < number_image_list; i++)
+      for (i=0; i < number_images; i++)
         image_list[i]=DestroyString(image_list[i]);
       image_list=(char **) RelinquishMagickMemory(image_list);
       return(images);
@@ -725,7 +725,8 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
                   Image
                     *size_image;
 
-                  flags=ParseSizeGeometry(next,read_info->extract,&geometry);
+                  flags=ParseSizeGeometry(next,read_info->extract,&geometry,
+                    exception);
                   size_image=ResizeImage(next,geometry.width,geometry.height,
                     next->filter,next->blur,exception);
                   if (size_image != (Image *) NULL)
@@ -816,7 +817,7 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
         **image_list;
 
       int
-        number_image_list;
+        number_images;
 
       register long
         i;
@@ -828,11 +829,11 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
       if (command == (char *) NULL)
         return((Image *) NULL);
       StripString(command);
-      image_list=StringToArgv(command,&number_image_list);
+      image_list=StringToArgv(command,&number_images);
       command=DestroyString(command);
       read_info=CloneImageInfo(image_info);
       images=NewImageList();
-      for (i=1; i < number_image_list; i++)
+      for (i=1; i < number_images; i++)
       {
         (void) CopyMagickString(read_info->filename,image_list[i],
           MaxTextExtent);
@@ -842,7 +843,7 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
         AppendImageToList(&images,image);
       }
       read_info=DestroyImageInfo(read_info);
-      for (i=1; i < number_image_list; i++)
+      for (i=0; i < number_images; i++)
         image_list[i]=DestroyString(image_list[i]);
       image_list=(char **) RelinquishMagickMemory(image_list);
       return(images);
