@@ -17,7 +17,7 @@
 %                                July 1993                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -290,7 +290,8 @@ static void DefaultErrorHandler(const ExceptionType magick_unused(severity),
 %    o description: Specifies any description to the reason.
 %
 */
-static void DefaultFatalErrorHandler(const ExceptionType magick_unused(severity),
+static void DefaultFatalErrorHandler(
+  const ExceptionType magick_unused(severity),
   const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
@@ -917,6 +918,8 @@ MagickExport MagickBooleanType ThrowMagickExceptionList(
   const char *format,va_list operands)
 {
   char
+    message[MaxTextExtent],
+    path[MaxTextExtent],
     reason[MaxTextExtent];
 
   const char
@@ -945,7 +948,10 @@ MagickExport MagickBooleanType ThrowMagickExceptionList(
   if (n < 0)
     reason[MaxTextExtent-1]='\0';
   status=LogMagickEvent(ExceptionEvent,module,function,line,"%s",reason);
-  (void) ThrowException(exception,severity,reason,(char *) NULL);
+  GetPathComponent(module,TailPath,path);
+  (void) FormatMagickString(message,MaxTextExtent,"%s @ %s/%s/%ld",reason,path,
+    function,line);
+  (void) ThrowException(exception,severity,message,(char *) NULL);
   return(status);
 }
 
