@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,15 +13,17 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image f/x methods.
+  MagickCore image f/x methods.
 */
-#ifndef _MAGICK_FX_H
-#define _MAGICK_FX_H
+#ifndef _MAGICKCORE_FX_H
+#define _MAGICKCORE_FX_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
+#include "magick/draw.h"
+
 typedef enum
 {
   UndefinedEvaluateOperator,
@@ -36,29 +38,62 @@ typedef enum
   RightShiftEvaluateOperator,
   SetEvaluateOperator,
   SubtractEvaluateOperator,
-  XorEvaluateOperator
+  XorEvaluateOperator,
+  PowEvaluateOperator,
+  LogEvaluateOperator,
+  ThresholdEvaluateOperator,
+  ThresholdBlackEvaluateOperator,
+  ThresholdWhiteEvaluateOperator,
+  GaussianNoiseEvaluateOperator,
+  ImpulseNoiseEvaluateOperator,
+  LaplacianNoiseEvaluateOperator,
+  MultiplicativeNoiseEvaluateOperator,
+  PoissonNoiseEvaluateOperator,
+  UniformNoiseEvaluateOperator
 } MagickEvaluateOperator;
 
+typedef enum
+{
+  UndefinedNoise,
+  UniformNoise,
+  GaussianNoise,
+  MultiplicativeGaussianNoise,
+  ImpulseNoise,
+  LaplacianNoise,
+  PoissonNoise,
+  RandomNoise
+} NoiseType;
+
 extern MagickExport Image
+  *AddNoiseImage(const Image *,const NoiseType,ExceptionInfo *),
+  *AddNoiseImageChannel(const Image *,const ChannelType,const NoiseType,
+    ExceptionInfo *),
   *CharcoalImage(const Image *,const double,const double,ExceptionInfo *),
   *ColorizeImage(const Image *,const char *,const PixelPacket,ExceptionInfo *),
   *ConvolveImage(const Image *,const unsigned long,const double *,
     ExceptionInfo *),
-  *ConvolveImageChannel(const Image *,ChannelType,const unsigned long,
+  *ConvolveImageChannel(const Image *,const ChannelType,const unsigned long,
     const double *,ExceptionInfo *),
-  *EdgeImage(const Image *,const double,ExceptionInfo *),
   *FxImage(const Image *,const char *,ExceptionInfo *),
   *FxImageChannel(const Image *,const ChannelType,const char *,ExceptionInfo *),
   *ImplodeImage(const Image *,const double,ExceptionInfo *),
   *MorphImages(const Image *,const unsigned long,ExceptionInfo *),
-  *OilPaintImage(const Image *,const double,ExceptionInfo *),
+  *PolaroidImage(const Image *,const DrawInfo *,const double,ExceptionInfo *),
+  *RecolorImage(const Image *,const unsigned long,const double *,
+    ExceptionInfo *),
   *SepiaToneImage(const Image *,const double,ExceptionInfo *),
   *ShadowImage(const Image *,const double,const double,const long,const long,
     ExceptionInfo *),
+  *SketchImage(const Image *,const double,const double,const double,
+    ExceptionInfo *),
   *SteganoImage(const Image *,const Image *,ExceptionInfo *),
   *StereoImage(const Image *,const Image *,ExceptionInfo *),
+  *StereoAnaglyphImage(const Image *,const Image *,const long,const long,
+     ExceptionInfo *),
   *SwirlImage(const Image *,double,ExceptionInfo *),
   *TintImage(const Image *,const char *,const PixelPacket,ExceptionInfo *),
+  *VignetteImage(const Image *,const double,const double,const long,
+    const long,ExceptionInfo *),
   *WaveImage(const Image *,const double,const double,ExceptionInfo *);
 
 extern MagickExport MagickBooleanType
