@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,49 +13,35 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  MagickCore graphic gems methods.
+  MagickCore private graphic gems methods.
 */
-#ifndef _MAGICKCORE_GEM_H
-#define _MAGICKCORE_GEM_H
+#ifndef _MAGICKCORE_GEM_PRIVATE_H
+#define _MAGICKCORE_GEM_PRIVATE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-#include <magick/pixel.h>
-
 extern MagickExport double
   ExpandAffine(const AffineMatrix *);
 
-extern MagickExport MagickPixelPacket
-  InterpolateColor(const Image *,const double,const double,ExceptionInfo *);
-
 extern MagickExport unsigned long
   GetOptimalKernelWidth(const double,const double),
   GetOptimalKernelWidth1D(const double,const double),
   GetOptimalKernelWidth2D(const double,const double);
 
 extern MagickExport void
-  Contrast(const int,Quantum *,Quantum *,Quantum *),
-  HSBTransform(const double,const double,const double,Quantum *,Quantum *,
+  ConvertHSBToRGB(const double,const double,const double,Quantum *,Quantum *,
     Quantum *),
-  HSLTransform(const double,const double,const double,Quantum *,Quantum *,
+  ConvertHSLToRGB(const double,const double,const double,Quantum *,Quantum *,
     Quantum *),
-  HWBTransform(const double,const double,const double,Quantum *,Quantum *,
+  ConvertHWBToRGB(const double,const double,const double,Quantum *,Quantum *,
     Quantum *),
-  Hull(const long,const long,const unsigned long,const unsigned long,Quantum *,
-    Quantum *,const int),
-  ModulateHSB(const double,const double,const double,Quantum *,Quantum *,
-    Quantum *),
-  ModulateHSL(const double,const double,const double,Quantum *,Quantum *,
-    Quantum *),
-  ModulateHWB(const double,const double,const double,Quantum *,Quantum *,
-    Quantum *),
-  TransformHSB(const Quantum,const Quantum,const Quantum,double *,double *,
+  ConvertRGBToHSB(const Quantum,const Quantum,const Quantum,double *,double *,
     double *),
-  TransformHSL(const Quantum,const Quantum,const Quantum,double *,double *,
+  ConvertRGBToHSL(const Quantum,const Quantum,const Quantum,double *,double *,
     double *),
-  TransformHWB(const Quantum,const Quantum,const Quantum,double *,double *,
+  ConvertRGBToHWB(const Quantum,const Quantum,const Quantum,double *,double *,
     double *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
