@@ -17,7 +17,7 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -89,6 +89,9 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
     pedantic_geometry[MaxTextExtent],
     *q;
 
+  double
+    value;
+
   MagickStatusType
     flags;
 
@@ -184,9 +187,9 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
   if (*p == '\0')
     return(flags);
   q=p;
-  (void) strtod(p,&q);
+  value=strtod(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
-    (void) strtol(p,&q,10);
+    value=(double) strtol(p,&q,10);
   if (((int) *q == -41) || (*q == 'x') || (*q == 'X') || (*q == '\0'))
     {
       /*
@@ -417,6 +420,10 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 MagickExport void GravityAdjustGeometry(const unsigned long width,
   const unsigned long height,const GravityType gravity,RectangleInfo *region)
 {
+  if (region->height == 0)
+    region->height=height;
+  if (region->width == 0)
+    region->width=width;
   switch (gravity)
   {
     case NorthEastGravity:
@@ -502,6 +509,8 @@ MagickExport MagickBooleanType IsGeometry(const char *geometry)
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(geometry,&geometry_info);
+  if (flags == NoValue)
+    flags=ParseGeometry(geometry+1,&geometry_info);  /* i.e. +-4+-4 */
   return(flags != NoValue ? MagickTrue : MagickFalse);
 }
 
@@ -538,10 +547,13 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
   char
     *p;
 
+  double
+    value;
+
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   p=(char *) geometry;
-  (void) strtod(geometry,&p);
+  value=strtod(geometry,&p);
   if (p == geometry)
     return(MagickFalse);
   if (strspn(geometry,"0123456789-, ") != strlen(geometry))
@@ -604,7 +616,7 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %  The format of the ParseAffineGeometry method is:
 %
 %      MagickStatusType ParseAffineGeometry(const char *geometry,
-%        AffineMatrix *affine_matrix)
+%        AffineMatrix *affine_matrix,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -612,9 +624,11 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %
 %    o affine_matrix: the affine matrix as defined by the geometry string.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
-  AffineMatrix *affine_matrix)
+  AffineMatrix *affine_matrix,ExceptionInfo *exception)
 {
   char
     token[MaxTextExtent];
@@ -622,6 +636,9 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
   const char
     *p;
 
+  double
+    determinant;
+
   MagickStatusType
     flags;
 
@@ -631,7 +648,7 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
   GetAffineMatrix(affine_matrix);
   flags=NoValue;
   p=(char *) geometry;
-  for (i=0; i < 6; i++)
+  for (i=0; (*p != '\0') && (i < 6); i++)
   {
     GetMagickToken(p,&p,token);
     if (*token == ',')
@@ -646,6 +663,11 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
       case 5: affine_matrix->ty=atof(token); flags|=YValue; break;
     }
   }
+  determinant=(affine_matrix->sx*affine_matrix->sy-affine_matrix->rx*
+    affine_matrix->ry);
+  if (fabs(determinant) < MagickEpsilon)
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+      "InvalidGeometry","`%s'",geometry);
   return(flags);
 }
 
@@ -792,9 +814,9 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   if (*p == '\0')
     return(flags);
   q=p;
-  (void) strtod(p,&q);
+  value=strtod(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
-    (void) strtol(p,&q,10);
+    value=(double) strtol(p,&q,10);
   if (((int) *q == -41) || (*q == 'x') || (*q == 'X') || (*q == ',') ||
       (*q == '/') || (*q == ':') || (*q =='\0'))
     {
@@ -923,7 +945,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %  The format of the ParseGravityGeometry method is:
 %
 %      MagickStatusType ParseGravityGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info)
+%        RectangeInfo *region_info,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -932,9 +954,11 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %    o region_info: the region as defined by the geometry string with
 %      respect to the image dimensions and its gravity.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-MagickExport MagickStatusType ParseGravityGeometry(Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
+  const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
     flags;
@@ -951,8 +975,8 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
   flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        OptionError,"InvalidGeometry","`%s'",geometry);
+      (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+        "InvalidGeometry","`%s'",geometry);
       return(flags);
     }
   if ((flags & PercentValue) != 0)
@@ -979,22 +1003,18 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
       if ((status & SigmaValue) == 0)
         scale.y=scale.x;
       region_info->width=(unsigned long) ((scale.x*image->columns/100.0)+0.5);
-      if (region_info->width == 0)
-        region_info->width=1;
       region_info->height=(unsigned long) ((scale.y*image->rows/100.0)+0.5);
-      if (region_info->height == 0)
-        region_info->height=1;
     }
   /*
     If a region width of 0 is given by the user, use the images virtual canvas
-    size for the gravity offset adjustments, then restore afterward.
+    size for the gravity offset adjustments-- restore afterward.
   */
   width=region_info->width;
   height=region_info->height;
   if (width == 0)
-    region_info->width  = image->page.width  | image->columns;
+    region_info->width=image->page.width | image->columns;
   if (height == 0)
-    region_info->height = image->page.height | image->rows;
+    region_info->height=image->page.height | image->rows;
   GravityAdjustGeometry(image->columns,image->rows,image->gravity,region_info);
   region_info->width=width;
   region_info->height=height;
@@ -1208,8 +1228,9 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
 %
 %  The format of the ParsePageGeometry method is:
 %
-%      MagickStatusType ParsePageGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info)
+%      MagickStatusType ParsePageGeometry(const Image *image,
+%        const char *geometry,RectangeInfo *region_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1218,9 +1239,11 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
 %    o region_info: the region as defined by the geometry string with
 %      respect to the image and its gravity.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-MagickExport MagickStatusType ParsePageGeometry(Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParsePageGeometry(const Image *image,
+  const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
     flags;
@@ -1233,8 +1256,8 @@ MagickExport MagickStatusType ParsePageGeometry(Image *image,
   flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        OptionError,"InvalidGeometry","`%s'",geometry);
+      (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+        "InvalidGeometry","`%s'",geometry);
       return(flags);
     }
   if ((flags & PercentValue) != 0)
@@ -1263,8 +1286,9 @@ MagickExport MagickStatusType ParsePageGeometry(Image *image,
 %
 %  The format of the ParseSizeGeometry method is:
 %
-%      MagickStatusType ParseSizeGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info)
+%      MagickStatusType ParseSizeGeometry(const Image *image,
+%        const char *geometry,RectangeInfo *region_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1272,9 +1296,11 @@ MagickExport MagickStatusType ParsePageGeometry(Image *image,
 %
 %    o region_info: the region as defined by the geometry string.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-MagickExport MagickStatusType ParseSizeGeometry(Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParseSizeGeometry(const Image *image,
+  const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
     flags;
@@ -1283,7 +1309,7 @@ MagickExport MagickStatusType ParseSizeGeometry(Image *image,
   flags=ParseMetaGeometry(geometry,&region_info->x,&region_info->y,
     &region_info->width,&region_info->height);
   if (flags == NoValue)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
       "InvalidGeometry","`%s'",geometry);
   return(flags);
 }
