@@ -17,7 +17,7 @@
 %                                September 2002                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -747,7 +747,7 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 
 static unsigned long GetLogThreadId(void)
 {
-#if defined(MAGICKCORE_HAVE_PTHREAD) && defined(MAGICKCORE_HAVE_LINUX_UNISTD_H)
+#if defined(MAGICKCORE_HAVE_PTHREAD) && defined(MAGICKCORE_HAVE_LINUX_UNISTD_H) && defined(__NR_gettid)
   return((unsigned long) ((pid_t) syscall(__NR_gettid)));
 #endif
 #if defined(__WINDOWS__)
@@ -1333,7 +1333,10 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
                   if (*path != '\0')
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  if (*token == *DirectorySeparator)
+                    (void) CopyMagickString(path,token,MaxTextExtent);
+                  else
+                    (void) ConcatenateMagickString(path,token,MaxTextExtent);
                   xml=FileToString(path,~0,exception);
                   if (xml != (char *) NULL)
                     {
