@@ -18,7 +18,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -1273,7 +1273,7 @@ MagickExport void MagickCoreTerminus(void)
   DestroyCoderList();
   DestroyMagickResources();
   DestroyImageRegistry();
-  DestroyCacheResources();
+  DestroyPixelCacheResources();
   DestroyRandomReservoir();
   DestroyLocaleList();
   DestroyLogList();
