@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@ extern "C" {
 #endif
 
 #include "magick/delegate.h"
+#include "magick/delegate-private.h"
 #include "magick/exception.h"
 
 #define WIN32_LEAN_AND_MEAN
@@ -81,6 +82,18 @@ extern "C" {
 #if !defined(fileno)
 #  define fileno  _fileno
 #endif
+#if !defined(fseek)
+#  define fseeko  _fseeki64
+#endif
+#if !defined(fstat)
+#  define fstat  _fstat64
+#endif
+#if !defined(fsync)
+#  define fsync  _commit
+#endif
+#if !defined(ftell)
+#  define ftello  _ftelli64
+#endif
 #if !defined(ftruncate)
 #  define ftruncate(file,length)  NTTruncateFile(file,length)
 #endif
@@ -93,9 +106,13 @@ extern "C" {
 #if !defined(hypot)
 #  define hypot  _hypot
 #endif
+#if !defined(inline)
+#  define inline __inline
+#endif
 #if !defined(isatty)
 #  define isatty _isatty
 #endif
+#if !defined(MAGICKCORE_LTDL_DELEGATE)
 #if !defined(lt_dlclose)
 #  define lt_dlclose(handle)  NTCloseLibrary(handle)
 #endif
@@ -117,6 +134,10 @@ extern "C" {
 #if !defined(lt_dlsym)
 #  define lt_dlsym(handle,name)  NTGetLibrarySymbol(handle,name)
 #endif
+#endif
+#if !defined(mkdir)
+#  define mkdir  _mkdir
+#endif
 #if !defined(mmap)
 #  define mmap(address,length,protection,access,file,offset) \
   NTMapMemory(address,length,protection,access,file,offset)
@@ -151,6 +172,9 @@ extern "C" {
 #if !defined(setmode)
 #  define setmode  _setmode
 #endif
+#if !defined(stat)
+#  define stat  __stat64
+#endif
 #if !defined(strcasecmp)
 #  define strcasecmp  _strcmpi
 #endif
@@ -167,7 +191,9 @@ extern "C" {
 #  define tempnam  _tempnam
 #endif
 #if !defined(vsnprintf)
-#  define vsnprintf  _vsnprintf
+#if !defined(_MSC_VER) || (defined(_MSC_VER) && _MSC_VER < 1500)
+#define vsnprintf _vsnprintf 
+#endif
 #endif
 #if !defined(write)
 #  define write  _write
@@ -243,20 +269,26 @@ typedef UINT
 typedef long ssize_t;
 #endif
 
+#if defined(MAGICKCORE_BZLIB_DELEGATE)
+#  if defined(_WIN32)
+#    define BZ_IMPORT 1
+#  endif
+#endif
+
 extern MagickExport char
   *NTGetLastError(void);
 
 extern MagickExport const GhostscriptVectors
   *NTGhostscriptDLLVectors(void);
 
-#if !defined(HasLTDL)
+#if !defined(MAGICKCORE_LTDL_DELEGATE)
 extern MagickExport const char
   *NTGetLibraryError(void);
 #endif
 
 #if !defined(XS_VERSION)
-extern MagickExport char
-  *NTRegistryKeyLookup(const char *);
+extern MagickExport const char
+  *NTGetLibraryError(void);
 
 extern MagickExport DIR
   *NTOpenDirectory(const char *);
@@ -272,7 +304,7 @@ extern MagickExport int
   NTCloseLibrary(void *),
   NTControlHandler(void),
   NTExitLibrary(void),
-  NTTruncateFile(int,const MagickSizeType),
+  NTTruncateFile(int,off_t),
   NTGhostscriptDLL(char *,int),
   NTGhostscriptEXE(char *,int),
   NTGhostscriptFonts(char *,int),
@@ -289,6 +321,7 @@ extern MagickExport long
   NTTellDirectory(DIR *);
 
 extern MagickExport MagickBooleanType
+  NTGatherRandomData(const size_t,unsigned char *),
   NTGetExecutionPath(char *,const size_t),
   NTGetModulePath(const char *,char *),
   NTReportEvent(const char *,const MagickBooleanType),
@@ -298,6 +331,7 @@ extern MagickExport struct dirent
   *NTReadDirectory(DIR *);
 
 extern MagickExport unsigned char
+  *NTRegistryKeyLookup(const char *),
   *NTResourceToBlob(const char *);
 
 extern MagickExport void
