@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -722,20 +722,60 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
 #define EXIF_FMT_SINGLE  11
 #define EXIF_FMT_DOUBLE  12
 #define TAG_EXIF_OFFSET  0x8769
+#define TAG_GPS_OFFSET  0x8825
 #define TAG_INTEROP_OFFSET  0xa005
 
+#define EXIFMultipleValues(size, format, arg) \
+{ \
+    long component; \
+    size_t used_space; \
+    unsigned char *p1; \
+    used_space=0; \
+    p1=p; \
+    for (component = 0; component < components; component++) \
+    { \
+       used_space+=FormatMagickString(buffer+used_space, \
+          MaxTextExtent-used_space,format", ",arg); \
+       if (used_space >= MaxTextExtent - 1) \
+          used_space=MaxTextExtent-1; \
+       p1+=size; \
+    } \
+    buffer[used_space-2]='\0'; \
+    value=AcquireString(buffer); \
+}
+
+#define EXIFMultipleFractions(size, format, arg1, arg2) \
+{ \
+    long component; \
+    size_t used_space; \
+    unsigned char *p1; \
+    used_space=0; \
+    p1=p; \
+    for (component = 0; component < components; component++) \
+    { \
+       used_space+=FormatMagickString(buffer+used_space, \
+          MaxTextExtent-used_space,format", ",arg1, arg2); \
+       if (used_space >= MaxTextExtent - 1) \
+          used_space=MaxTextExtent-1; \
+       p1+=size; \
+    } \
+    buffer[used_space-2]='\0'; \
+    value=AcquireString(buffer); \
+}
+
   typedef struct _DirectoryInfo
   {
     const unsigned char
       *directory;
 
     unsigned long
-      entry;
+      entry,
+      offset;
   } DirectoryInfo;
 
   typedef struct _TagInfo
   {
-    unsigned short
+    unsigned long
       tag;
 
     const char
@@ -743,7 +783,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
   } TagInfo;
 
   static TagInfo
-    Tags[] =
+    EXIFTag[] =
     {
       {  0x001, "exif:InteroperabilityIndex" },
       {  0x002, "exif:InteroperabilityVersion" },
@@ -982,6 +1022,37 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       {  0xa40c, "exif:SubjectDistanceRange" },
       {  0xa420, "exif:ImageUniqueID" },
       {  0xc4a5, "exif:PrintImageMatching" },
+      { 0x10000, "exif:GPSVersionID" },
+      { 0x10001, "exif:GPSLatitudeRef" },
+      { 0x10002, "exif:GPSLatitude" },
+      { 0x10003, "exif:GPSLongitudeRef" },
+      { 0x10004, "exif:GPSLongitude" },
+      { 0x10005, "exif:GPSAltitudeRef" },
+      { 0x10006, "exif:GPSAltitude" },
+      { 0x10007, "exif:GPSTimeStamp" },
+      { 0x10008, "exif:GPSSatellites" },
+      { 0x10009, "exif:GPSStatus" },
+      { 0x1000a, "exif:GPSMeasureMode" },
+      { 0x1000b, "exif:GPSDop" },
+      { 0x1000c, "exif:GPSSpeedRef" },
+      { 0x1000d, "exif:GPSSpeed" },
+      { 0x1000e, "exif:GPSTrackRef" },
+      { 0x1000f, "exif:GPSTrack" },
+      { 0x10010, "exif:GPSImgDirectionRef" },
+      { 0x10011, "exif:GPSImgDirection" },
+      { 0x10012, "exif:GPSMapDatum" },
+      { 0x10013, "exif:GPSDestLatitudeRef" },
+      { 0x10014, "exif:GPSDestLatitude" },
+      { 0x10015, "exif:GPSDestLongitudeRef" },
+      { 0x10016, "exif:GPSDestLongitude" },
+      { 0x10017, "exif:GPSDestBearingRef" },
+      { 0x10018, "exif:GPSDestBearing" },
+      { 0x10019, "exif:GPSDestDistanceRef" },
+      { 0x1001a, "exif:GPSDestDistance" },
+      { 0x1001b, "exif:GPSProcessingMethod" },
+      { 0x1001c, "exif:GPSAreaInformation" },
+      { 0x1001d, "exif:GPSDateStamp" },
+      { 0x1001e, "exif:GPSDifferential" },
       {  0x0000, NULL}
     };
 
@@ -1019,6 +1090,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
   unsigned long
     entry,
     number_entries,
+    tag_offset,
     tag;
 
   /*
@@ -1051,6 +1123,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       break;
     }
     case '#':
+    case '@':
     {
       int
         c;
@@ -1061,7 +1134,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       /*
         Check for a hex based tag specification first.
       */
-      tag=0;
+      tag=(*(property+5) == '@') ? 1 : 0;
       property+=6;
       n=strlen(property);
       if (n != 4)
@@ -1097,11 +1170,11 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       */
       for (i=0; ; i++)
       {
-        if (Tags[i].tag == 0)
+        if (EXIFTag[i].tag == 0)
           break;
-        if (LocaleCompare(Tags[i].description,property) == 0)
+        if (LocaleCompare(EXIFTag[i].description,property) == 0)
           {
-            tag=(unsigned long) Tags[i].tag;
+            tag=(unsigned long) EXIFTag[i].tag;
             break;
           }
       }
@@ -1153,6 +1226,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
   directory=exif+offset;
   level=0;
   entry=0;
+  tag_offset=0;
   do
   {
     /*
@@ -1163,6 +1237,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
         level--;
         directory=directory_stack[level].directory;
         entry=directory_stack[level].entry;
+        tag_offset=directory_stack[level].offset;
       }
     /*
       Determine how many entries there are in the current IFD.
@@ -1184,7 +1259,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
         format;
 
       q=(unsigned char *) (directory+2+(12*entry));
-      tag_value=(long) ReadPropertyShort(endian,q);
+      tag_value=(long) ReadPropertyShort(endian,q) + tag_offset;
       format=(unsigned long) ReadPropertyShort(endian,q+2);
       if (format >= (sizeof(tag_bytes)/sizeof(*tag_bytes)))
         break;
@@ -1213,72 +1288,61 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
 
           switch (format)
           {
+            case EXIF_FMT_BYTE:
+            case EXIF_FMT_UNDEFINED:
+            {
+              EXIFMultipleValues(1,"%lu",(unsigned long)
+                (*(unsigned char *) p1));
+              break;
+            }
             case EXIF_FMT_SBYTE:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%ld",
-                (long) (*(char *) p));
-              value=AcquireString(buffer);
+              EXIFMultipleValues(1,"%ld",(long) (*(signed char *) p1));
               break;
             }
             case EXIF_FMT_SSHORT:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%hd",
-                ReadPropertyShort(endian,p));
-              value=AcquireString(buffer);
+              EXIFMultipleValues(2,"%hd",ReadPropertyShort(endian,p1));
               break;
             }
             case EXIF_FMT_USHORT:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%hu",
-                ReadPropertyShort(endian,p));
-              value=AcquireString(buffer);
+              EXIFMultipleValues(2,"%hu",ReadPropertyShort(endian,p1));
               break;
             }
             case EXIF_FMT_ULONG:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%lu",
-                ReadPropertyLong(endian,p));
-              value=AcquireString(buffer);
+              EXIFMultipleValues(4,"%lu",ReadPropertyLong(endian,p1));
               break;
             }
             case EXIF_FMT_SLONG:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%ld",
-                ReadPropertyLong(endian,p));
-              value=AcquireString(buffer);
+              EXIFMultipleValues(4,"%ld",ReadPropertyLong(endian,p1));
               break;
             }
             case EXIF_FMT_URATIONAL:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%ld/%ld",
-                ReadPropertyLong(endian,p),ReadPropertyLong(endian,p+4));
-              value=AcquireString(buffer);
+              EXIFMultipleFractions(8,"%ld/%ld",ReadPropertyLong(endian,p1),
+                ReadPropertyLong(endian,p1+4));
               break;
             }
             case EXIF_FMT_SRATIONAL:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%ld/%ld",
-                ReadPropertyLong(endian,p),ReadPropertyLong(endian,p+4));
-              value=AcquireString(buffer);
+              EXIFMultipleFractions(8,"%ld/%ld",ReadPropertyLong(endian,p1),
+                ReadPropertyLong(endian,p1+4));
               break;
             }
             case EXIF_FMT_SINGLE:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%f",
-                (double) *(float *) p);
-              value=AcquireString(buffer);
+              EXIFMultipleValues(4,"%f",(double) *(float *) p1);
               break;
             }
             case EXIF_FMT_DOUBLE:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"%f",
-                *(double *) p);
-              value=AcquireString(buffer);
+              EXIFMultipleValues(8,"%f",*(double *) p1);
               break;
             }
             default:
-            case EXIF_FMT_UNDEFINED:
-            case EXIF_FMT_BYTE:
             case EXIF_FMT_STRING:
             {
               value=(char *) NULL;
@@ -1323,11 +1387,11 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
                   description="unknown";
                   for (i=0; ; i++)
                   {
-                    if (Tags[i].tag == 0)
+                    if (EXIFTag[i].tag == 0)
                       break;
-                    if ((long) Tags[i].tag == tag_value)
+                    if ((long) EXIFTag[i].tag == tag_value)
                       {
-                        description=Tags[i].description;
+                        description=EXIFTag[i].description;
                         break;
                       }
                   }
@@ -1337,8 +1401,15 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
                 }
                 case 2:
                 {
-                  (void) FormatMagickString(key,MaxTextExtent,"#%04lx",
-                    tag_value);
+                  if (tag_value < 0x10000)
+                    (void) FormatMagickString(key,MaxTextExtent,"#%04lx",
+                      tag_value);
+                  else
+                    if (tag_value < 0x20000)
+                      (void) FormatMagickString(key,MaxTextExtent,"@%04lx",
+                        tag_value & 0xffff);
+                    else
+                      (void) FormatMagickString(key,MaxTextExtent,"unknown");
                   break;
                 }
               }
@@ -1349,7 +1420,9 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
               value=DestroyString(value);
             }
         }
-        if ((tag_value == TAG_EXIF_OFFSET) || (tag_value == TAG_INTEROP_OFFSET))
+        if ((tag_value == TAG_EXIF_OFFSET)
+         || (tag_value == TAG_INTEROP_OFFSET)
+         || (tag_value == TAG_GPS_OFFSET))
           {
             size_t
               offset;
@@ -1357,11 +1430,17 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
             offset=(size_t) ReadPropertyLong(endian,p);
             if ((offset < length) && (level < (MaxDirectoryStack-2)))
               {
+                unsigned long
+                  tag_offset1;
+
+                tag_offset1=(tag_value == TAG_GPS_OFFSET) ? 0x10000 : 0;
                 directory_stack[level].directory=directory;
                 entry++;
                 directory_stack[level].entry=entry;
+                directory_stack[level].offset=tag_offset;
                 level++;
                 directory_stack[level].directory=exif+offset;
+                directory_stack[level].offset=tag_offset1;
                 directory_stack[level].entry=0;
                 level++;
                 if ((directory+2+(12*number_entries)) > (exif+length))
@@ -1373,6 +1452,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
                   {
                     directory_stack[level].directory=exif+offset;
                     directory_stack[level].entry=0;
+                    directory_stack[level].offset=tag_offset1;
                     level++;
                   }
               }
@@ -1386,6 +1466,9 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
 static MagickBooleanType GetXMPProperty(const Image *image,
   const char *property)
 {
+  char
+    *xmp_profile;
+
   const StringInfo
     *profile;
 
@@ -1398,9 +1481,6 @@ static MagickBooleanType GetXMPProperty(const Image *image,
   register const char
     *p;
 
-  register long
-    i;
-
   XMLTreeInfo
     *child,
     *description,
@@ -1413,15 +1493,15 @@ static MagickBooleanType GetXMPProperty(const Image *image,
     return(MagickFalse);
   if ((property == (const char *) NULL) || (*property == '\0'))
     return(MagickFalse);
-  exception=AcquireExceptionInfo();
-  p=(const char *) GetStringInfoDatum(profile);
-  for (i=0; i < (long) (GetStringInfoLength(profile)-1); i++)
-  {
+  xmp_profile=StringInfoToString(profile);
+  if (xmp_profile == (char *) NULL)
+    return(MagickFalse);
+  for (p=xmp_profile; *p != '\0'; p++)
     if ((*p == '<') && (*(p+1) == 'x'))
       break;
-    p++;
-  }
+  exception=AcquireExceptionInfo();
   xmp=NewXMLTree((char *) p,exception);
+  xmp_profile=DestroyString(xmp_profile);
   exception=DestroyExceptionInfo(exception);
   if (xmp == (XMLTreeInfo *) NULL)
     return(MagickFalse);
@@ -2628,8 +2708,8 @@ MagickExport char *InterpretImageProperties(const ImageInfo *image_info,
         colorspace=image->colorspace;
         if (IsGrayImage(image,&image->exception) != MagickFalse)
           colorspace=GRAYColorspace;
-        q+=FormatMagickString(q,extent,"%s%s%s",
-          MagickOptionToMnemonic(MagickClassOptions,(long) image->storage_class),
+        q+=FormatMagickString(q,extent,"%s%s%s",MagickOptionToMnemonic(
+          MagickClassOptions,(long) image->storage_class),
           MagickOptionToMnemonic(MagickColorspaceOptions,(long) colorspace),
           image->matte != MagickFalse ? "Matte" : "");
         break;
