@@ -281,7 +281,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
   assert(quantum_info != (QuantumInfo *) NULL);
   assert(quantum_info->signature == MagickSignature);
   if (pixels == (const unsigned char *) NULL)
-    pixels=quantum_info->pixels;
+    pixels=GetQuantumPixels(quantum_info);
   x=0;
   p=pixels;
   number_pixels=GetImageExtent(image);
@@ -532,13 +532,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           register unsigned char
             pixel;
 
+          scale=GetQuantumScale(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             pixel=(unsigned char) ((*p >> 4) & 0xf);
             indexes[x]=PushColormapIndex(image,pixel,&range_exception);
             *q=image->colormap[(long) indexes[x]];
-            q->opacity=(Quantum) (QuantumRange-(QuantumRange*((int) (*p) &
-              0xf)/15));
+            pixel=(unsigned char) ((*p) & 0xf);
+            q->opacity=(Quantum) (QuantumRange-
+              ScaleAnyToQuantum(pixel,image->depth,scale));
             p++;
             q++;
           }
@@ -735,7 +737,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
               for (x=0; x < (long) number_pixels; x++)
               {
                 p=PushCharPixel(p,&pixel);
-                q->red=(Quantum) (QuantumRange-q->red);
+                q->red=(Quantum) (QuantumRange-ScaleCharToQuantum(pixel));
                 q->green=q->red;
                 q->blue=q->red;
                 p+=quantum_info->pad;
@@ -1012,7 +1014,9 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
             q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
             q->green=q->red;
             q->blue=q->red;
-            q->opacity=(Quantum) (QuantumRange-(QuantumRange*((*p) & 0xf)/15));
+            pixel=(unsigned char) ((*p) & 0xf);
+            q->opacity=(Quantum) (QuantumRange-
+              ScaleAnyToQuantum(pixel,image->depth,scale));
             p++;
             q++;
           }
@@ -2515,6 +2519,87 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
       }
       break;
     }
+    case CbYCrYQuantum:
+    {
+      switch (quantum_info->depth)
+      {
+        case 10:
+        {
+          Quantum
+            cbcr[4];
+
+          pixel=0;
+          if (quantum_info->pack == MagickFalse)
+            {
+              long
+                n;
+
+              register long
+                i;
+
+              unsigned long
+                quantum;
+
+              n=0;
+              quantum=0;
+              for (x=0; x < (long) number_pixels; x+=2)
+              {
+                for (i=0; i < 4; i++)
+                {
+                  switch (n % 3)
+                  {
+                    case 0:
+                    {
+                      p=PushLongPixel(endian,p,&pixel);
+                      quantum=(unsigned long) (ScaleShortToQuantum(
+                        (unsigned short) (((pixel >> 22) & 0x3ff) << 6)));
+                      break;
+                    }
+                    case 1:
+                    {
+                      quantum=(unsigned long) (ScaleShortToQuantum(
+                        (unsigned short) (((pixel >> 12) & 0x3ff) << 6)));
+                      break;
+                    }
+                    case 2:
+                    {
+                      quantum=(unsigned long) (ScaleShortToQuantum(
+                        (unsigned short) (((pixel >> 2) & 0x3ff) << 6)));
+                      break;
+                    }
+                  }
+                  cbcr[i]=(Quantum) (quantum);
+                  n++;
+                }
+                p+=quantum_info->pad;
+                q->red=cbcr[1];
+                q->green=cbcr[0];
+                q->blue=cbcr[2];
+                q++;
+                q->red=cbcr[3];
+                q->green=cbcr[0];
+                q->blue=cbcr[2];
+                q++;
+              }
+              break;
+            }
+        }
+        default:
+        {
+          scale=GetQuantumScale(image->depth);
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
+            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
+            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q++;
+          }
+          break;
+        }
+      }
+      break;
+    }
     default:
       break;
   }
