@@ -10,14 +10,14 @@
 %                      T     OOO   K   K  EEEEE  N   N                        %
 %                                                                             %
 %                                                                             %
-%                        ImageMagick Token Methods                            %
+%                         MagickCore Token Methods                            %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -164,6 +164,9 @@ MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
 */
 MagickExport void GetMagickToken(const char *start,const char **end,char *token)
 {
+  double
+    value;
+
   register const char
     *p;
 
@@ -221,7 +224,7 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
         char
           *q;
 
-        (void) strtod(p,&q);
+        value=strtod(p,&q);
         if (p != q)
           {
             for ( ; p < q; p++)
@@ -260,17 +263,17 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
     break;
   }
   token[i]='\0';
-  if (LocaleNCompare(token,"url(#",5) == 0)
-    {
-      i=(long) strlen(token);
-      (void) CopyMagickString(token,token+5,MaxTextExtent);
-      token[i-6]='\0';
-    }
   if (LocaleNCompare(token,"url(",4) == 0)
     {
+      ssize_t
+        offset;
+
+      offset=4;
+      if (token[offset] == '#')
+        offset++;
       i=(long) strlen(token);
-      (void) CopyMagickString(token,token+4,MaxTextExtent);
-      token[i-5]='\0';
+      (void) CopyMagickString(token,token+offset,MaxTextExtent);
+      token[i-offset-1]='\0';
     }
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
@@ -567,7 +570,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %
 %  A description of each parameter follows:
 %
-%    o path: The path.
+%    o path: the path.
 %
 */
 MagickExport MagickBooleanType IsGlob(const char *path)
@@ -575,6 +578,8 @@ MagickExport MagickBooleanType IsGlob(const char *path)
   MagickBooleanType
     status;
 
+  if (IsPathAccessible(path) != MagickFalse)
+    return(MagickFalse);
   status=(strchr(path,'*') != (char *) NULL) ||
     (strchr(path,'?') != (char *) NULL) ||
     (strchr(path,'{') != (char *) NULL) ||
