@@ -10,10 +10,10 @@ PACKAGE_NAME='ImageMagick'
 #
 # Package version.  This is is the numeric version suffix applied to
 # PACKAGE_NAME (e.g. "1.0.0").
-PACKAGE_VERSION='6.4.5'
-PACKAGE_LIB_VERSION="0x645"
-PACKAGE_LIB_VERSION_NUMBER="6,4,5,6"
-PACKAGE_RELEASE="6"
+PACKAGE_VERSION='6.4.8'
+PACKAGE_LIB_VERSION="0x648"
+PACKAGE_LIB_VERSION_NUMBER="6,4,8,3"
+PACKAGE_RELEASE="3"
 PACKAGE_RELEASE_DATE=`date +%F`
 
 #
@@ -23,13 +23,8 @@ PACKAGE_CHANGE_DATE=`awk '/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/ { print
 
 #
 # Package version addendum.  This is an arbitrary suffix (if any) appended
-# to the package version. (e.g. "beta1") `echo -snapshot-``date '+%g%m%d'`
-CHANGE_DATE=`find ${srcdir}/ChangeLog -printf '%Ty%Tm%Td%\n' 2> /dev/null`
-if test -n "$CHANGE_DATE"; then
-  PACKAGE_VERSION_ADDENDUM=""
-else
-  PACKAGE_VERSION_ADDENDUM=""
-fi
+# to the package version. (e.g. "beta1")
+PACKAGE_VERSION_ADDENDUM="-${PACKAGE_RELEASE}"
 
 #
 # Libtool library revision control info: See the libtool documentation under
