@@ -17,7 +17,7 @@
 %                                July 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -88,7 +88,7 @@
 %
 */
 
-static void AnimateUsage(void)
+static MagickBooleanType AnimateUsage(void)
 {
   const char
     **p;
@@ -216,7 +216,7 @@ static void AnimateUsage(void)
   (void) printf("\nButtons: \n");
   for (p=buttons; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
-  exit(0);
+  return(MagickTrue);
 }
 
 WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
@@ -247,7 +247,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
 #define ThrowAnimateInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyAnimate(); \
   return(MagickFalse); \
 }
@@ -351,7 +351,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
       }
     if ((LocaleCompare("help",option+1) == 0) ||
         (LocaleCompare("-help",option+1) == 0))
-      AnimateUsage();
+      return(AnimateUsage());
   }
   /*
     Get user defaults from X resource database.
@@ -564,6 +564,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
               clone_images=image_stack[k-1].image;
             if (clone_images == (Image *) NULL)
               ThrowAnimateException(ImageError,"ImageSequenceRequired",option);
+            FireImageStack(MagickFalse,MagickTrue,MagickTrue);
             if (*option == '+')
               clone_images=CloneImages(clone_images,"-1",exception);
             else
@@ -577,7 +578,6 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
               }
             if (clone_images == (Image *) NULL)
               ThrowAnimateException(OptionError,"NoSuchImage",option);
-            FireImageStack(MagickFalse,MagickTrue,MagickTrue);
             AppendImageStack(clone_images);
             break;
           }
@@ -947,6 +947,9 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -963,7 +966,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowAnimateException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowAnimateInvalidArgumentException(option,argv[i]);
             break;
@@ -1395,7 +1398,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
 #else
   (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
     "DelegateLibrarySupportNotBuiltIn","`%s' (X11)",image_info->filename);
-  AnimateUsage();
+  (void) AnimateUsage();
   return(MagickFalse);
 #endif
 }
