@@ -17,7 +17,7 @@
 %                            September 1994                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -84,7 +84,7 @@
 %
 */
 
-static void IdentifyUsage(void)
+static MagickBooleanType IdentifyUsage(void)
 {
   const char
     **p;
@@ -156,7 +156,7 @@ static void IdentifyUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  exit(0);
+  return(MagickTrue);
 }
 
 WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
@@ -179,7 +179,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
 #define ThrowIdentifyInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyIdentify(); \
   return(MagickFalse); \
 }
@@ -232,7 +232,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
         }
     }
   if (argc < 2)
-    IdentifyUsage();
+    return(IdentifyUsage());
   count=0;
   format=NULL;
   j=1;
@@ -249,6 +249,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
   if (status == MagickFalse)
     ThrowIdentifyException(ResourceLimitError,"MemoryAllocationFailed",
       strerror(errno));
+  image_info->ping=MagickTrue;
   for (i=1; i < (long) argc; i++)
   {
     option=argv[i];
@@ -525,7 +526,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
-          IdentifyUsage();
+          return(IdentifyUsage());
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
       }
       case 'i':
@@ -573,6 +574,9 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -589,7 +593,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowIdentifyException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
