@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -95,7 +95,7 @@
 %
 */
 
-static void ImportUsage(void)
+static MagickBooleanType ImportUsage(void)
 {
   const char
     **p;
@@ -209,7 +209,7 @@ static void ImportUsage(void)
   (void) printf(
     "the filename suffix (i.e. image.ps).  Specify 'file' as '-' for\n");
   (void) printf("standard input or output.\n");
-  exit(0);
+  return(MagickTrue);
 }
 
 static inline long MagickMax(const long x,const long y)
@@ -232,6 +232,8 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
       display=(Display *) NULL; \
     } \
   DestroyImageStack(); \
+  if (target_window != (char *) NULL) \
+    target_window=DestroyString(target_window); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
@@ -246,7 +248,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
 #define ThrowImportInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyImport(); \
   return(MagickFalse); \
 }
@@ -353,7 +355,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
       }
     if ((LocaleCompare("help",option+1) == 0) ||
         (LocaleCompare("-help",option+1) == 0))
-      ImportUsage();
+      return(ImportUsage());
   }
   /*
     Get user defaults from X resource database.
@@ -910,6 +912,9 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -926,7 +931,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowImportException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowImportInvalidArgumentException(option,argv[i]);
             break;
@@ -1265,7 +1270,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
 #else
   (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
     "DelegateLibrarySupportNotBuiltIn","`%s' (X11)",image_info->filename);
-  ImportUsage();
+  (void) ImportUsage();
   return(MagickFalse);
 #endif
 }
