@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -78,7 +78,7 @@
 %
 */
 
-static void MontageUsage(void)
+static MagickBooleanType MontageUsage(void)
 {
   const char
     **p;
@@ -235,7 +235,7 @@ static void MontageUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  exit(0);
+  return(MagickTrue);
 }
 
 WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
@@ -260,7 +260,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
 #define ThrowMontageInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyMontage(); \
   return(MagickFalse); \
 }
@@ -320,7 +320,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
         }
     }
   if (argc < 3)
-   MontageUsage();
+    return(MontageUsage());
   format="%w,%h,%m";
   first_scene=0;
   j=1;
@@ -587,6 +587,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               clone_images=image_stack[k-1].image;
             if (clone_images == (Image *) NULL)
               ThrowMontageException(ImageError,"ImageSequenceRequired",option);
+            FireImageStack(MagickTrue,MagickTrue,MagickTrue);
             if (*option == '+')
               clone_images=CloneImages(clone_images,"-1",exception);
             else
@@ -600,7 +601,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               }
             if (clone_images == (Image *) NULL)
               ThrowMontageException(OptionError,"NoSuchImage",option);
-            FireImageStack(MagickTrue,MagickTrue,MagickTrue);
             AppendImageStack(clone_images);
             break;
           }
@@ -961,7 +961,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
       {
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
-          MontageUsage();
+          return(MontageUsage());
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
       }
       case 'i':
@@ -1020,6 +1020,9 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -1036,7 +1039,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowMontageException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowMontageInvalidArgumentException(option,argv[i]);
             break;
