@@ -23,7 +23,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -74,11 +74,14 @@ struct _PixelView
     *wand;
 
   ViewInfo
-    **views;
+    *view;
 
   RectangleInfo
     region;
 
+  unsigned long
+    number_threads;
+
   PixelWand
     ***pixel_wands;
 
@@ -133,10 +136,10 @@ WandExport PixelView *ClonePixelView(const PixelView *pixel_view)
     clone_view->id);
   clone_view->exception=AcquireExceptionInfo();
   InheritException(clone_view->exception,pixel_view->exception);
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
-    clone_view->views[i]=CloneCacheView(pixel_view->views[i]);
+  clone_view->view=CloneCacheView(pixel_view->view);
   clone_view->region=pixel_view->region;
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  clone_view->number_threads=pixel_view->number_threads;
+  for (i=0; i < (long) pixel_view->number_threads; i++)
     clone_view->pixel_wands[i]=ClonePixelWands((const PixelWand **)
       pixel_view->pixel_wands[i],pixel_view->region.width);
   clone_view->debug=pixel_view->debug;
@@ -161,22 +164,27 @@ WandExport PixelView *ClonePixelView(const PixelView *pixel_view)
 %
 %  The format of the DestroyPixelView method is:
 %
-%      PixelView *DestroyPixelView(PixelView *pixel_view)
+%      PixelView *DestroyPixelView(PixelView *pixel_view,
+%        const unsigned long number_wands,const unsigned long number_threads)
 %
 %  A description of each parameter follows:
 %
 %    o pixel_view: the pixel view.
 %
+%    o number_wand: the number of pixel wands.
+%
+%    o number_threads: number of threads.
+%
 */
 
 static PixelWand ***DestroyPixelsThreadSet(PixelWand ***pixel_wands,
-  const unsigned long number_wands)
+  const unsigned long number_wands,const unsigned long number_threads)
 {
   register long
     i;
 
   assert(pixel_wands != (PixelWand ***) NULL);
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  for (i=0; i < (long) number_threads; i++)
     if (pixel_wands[i] != (PixelWand **) NULL)
       pixel_wands[i]=DestroyPixelWands(pixel_wands[i],number_wands);
   return((PixelWand ***) RelinquishMagickMemory(pixel_wands));
@@ -187,8 +195,8 @@ WandExport PixelView *DestroyPixelView(PixelView *pixel_view)
   assert(pixel_view != (PixelView *) NULL);
   assert(pixel_view->signature == WandSignature);
   pixel_view->pixel_wands=DestroyPixelsThreadSet(pixel_view->pixel_wands,
-    pixel_view->region.width);
-  pixel_view->views=DestroyCacheViewThreadSet(pixel_view->views);
+    pixel_view->region.width,pixel_view->number_threads);
+  pixel_view->view=DestroyCacheView(pixel_view->view);
   pixel_view->exception=DestroyExceptionInfo(pixel_view->exception);
   pixel_view->signature=(~WandSignature);
   RelinquishWandId(pixel_view->id);
@@ -274,9 +282,14 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
   status=MagickTrue;
   progress=0;
   exception=destination->exception;
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+#endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
+    MagickBooleanType
+      sync;
+
     register const IndexPacket
       *duplex_indexes,
       *indexes;
@@ -297,15 +310,15 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    pixels=GetCacheViewVirtualPixels(source->views[id],source->region.x,y,
+    id=GetPixelCacheThreadId();
+    pixels=GetCacheViewVirtualPixels(source->view,source->region.x,y,
       source->region.width,1,source->exception);
     if (pixels == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(source->views[id]);
+    indexes=GetCacheViewVirtualIndexQueue(source->view);
     for (x=0; x < (long) source->region.width; x++)
       PixelSetQuantumColor(source->pixel_wands[id][x],pixels+x);
     if (source_image->colorspace == CMYKColorspace)
@@ -314,14 +327,14 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
     if (source_image->storage_class == PseudoClass)
       for (x=0; x < (long) source->region.width; x++)
         PixelSetIndex(source->pixel_wands[id][x],indexes[x]);
-    duplex_pixels=GetCacheViewVirtualPixels(duplex->views[id],duplex->region.x,y,
+    duplex_pixels=GetCacheViewVirtualPixels(duplex->view,duplex->region.x,y,
       duplex->region.width,1,duplex->exception);
     if (duplex_pixels == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    duplex_indexes=GetCacheViewVirtualIndexes(duplex->views[id]);
+    duplex_indexes=GetCacheViewVirtualIndexQueue(duplex->view);
     for (x=0; x < (long) duplex->region.width; x++)
       PixelSetQuantumColor(duplex->pixel_wands[id][x],duplex_pixels+x);
     if (duplex_image->colorspace == CMYKColorspace)
@@ -330,14 +343,14 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
     if (duplex_image->storage_class == PseudoClass)
       for (x=0; x < (long) duplex->region.width; x++)
         PixelSetIndex(duplex->pixel_wands[id][x],duplex_indexes[x]);
-    destination_pixels=GetCacheViewAuthenticPixels(destination->views[id],
+    destination_pixels=GetCacheViewAuthenticPixels(destination->view,
       destination->region.x,y,destination->region.width,1,exception);
     if (destination_pixels == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    destination_indexes=GetCacheViewAuthenticIndexes(destination->views[id]);
+    destination_indexes=GetCacheViewAuthenticIndexQueue(destination->view);
     for (x=0; x < (long) destination->region.width; x++)
       PixelSetQuantumColor(destination->pixel_wands[id][x],
         destination_pixels+x);
@@ -357,10 +370,11 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
       for (x=0; x < (long) destination->region.width; x++)
         destination_indexes[x]=PixelGetBlackQuantum(
           destination->pixel_wands[id][x]);
-    if (SyncCacheViewAuthenticPixels(destination->views[id],exception) == MagickFalse)
+    sync=SyncCacheViewAuthenticPixels(destination->view,exception);
+    if (sync == MagickFalse)
       {
         InheritException(destination->exception,GetCacheViewException(
-          source->views[id]));
+          source->view));
         status=MagickFalse;
       }
     if (source_image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -368,7 +382,9 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(source_image,DuplexTransferPixelViewTag,
           progress++,source->region.height);
         if (proceed == MagickFalse)
@@ -526,7 +542,9 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
   source_image=source->wand->images;
   status=MagickTrue;
   progress=0;
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+#endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
     register const IndexPacket
@@ -541,15 +559,15 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    pixels=GetCacheViewVirtualPixels(source->views[id],source->region.x,y,
+    id=GetPixelCacheThreadId();
+    pixels=GetCacheViewVirtualPixels(source->view,source->region.x,y,
       source->region.width,1,source->exception);
     if (pixels == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(source->views[id]);
+    indexes=GetCacheViewVirtualIndexQueue(source->view);
     for (x=0; x < (long) source->region.width; x++)
       PixelSetQuantumColor(source->pixel_wands[id][x],pixels+x);
     if (source_image->colorspace == CMYKColorspace)
@@ -565,7 +583,9 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(source_image,GetPixelViewTag,progress++,
           source->region.height);
         if (proceed == MagickFalse)
@@ -599,9 +619,13 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
 */
 WandExport PixelWand **GetPixelViewPixels(const PixelView *pixel_view)
 {
+  long
+    id;
+
   assert(pixel_view != (PixelView *) NULL);
   assert(pixel_view->signature == WandSignature);
-  return(pixel_view->pixel_wands[GetCacheViewThreadId()]);
+  id=GetPixelCacheThreadId();
+  return(pixel_view->pixel_wands[id]);
 }
 
 /*
@@ -782,7 +806,8 @@ WandExport MagickBooleanType IsPixelView(const PixelView *pixel_view)
 %
 */
 
-static PixelWand ***AcquirePixelsThreadSet(const size_t number_wands)
+static PixelWand ***AcquirePixelsThreadSet(const unsigned long number_wands,
+  const unsigned long number_threads)
 {
   PixelWand
     ***pixel_wands;
@@ -790,17 +815,16 @@ static PixelWand ***AcquirePixelsThreadSet(const size_t number_wands)
   register long
     i;
 
-  pixel_wands=(PixelWand ***) AcquireQuantumMemory(
-    GetCacheViewMaximumThreads(),sizeof(*pixel_wands));
+  pixel_wands=(PixelWand ***) AcquireQuantumMemory(number_threads,
+    sizeof(*pixel_wands));
   if (pixel_wands == (PixelWand ***) NULL)
     return((PixelWand ***) NULL);
-  (void) ResetMagickMemory(pixel_wands,0,GetCacheViewMaximumThreads()*
-    sizeof(*pixel_wands));
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  (void) ResetMagickMemory(pixel_wands,0,number_threads*sizeof(*pixel_wands));
+  for (i=0; i < (long) number_threads; i++)
   {
     pixel_wands[i]=NewPixelWands(number_wands);
     if (pixel_wands[i] == (PixelWand **) NULL)
-      return(DestroyPixelsThreadSet(pixel_wands,number_wands));
+      return(DestroyPixelsThreadSet(pixel_wands,number_wands,number_threads));
   }
   return(pixel_wands);
 }
@@ -822,10 +846,12 @@ WandExport PixelView *NewPixelView(MagickWand *wand)
     PixelViewId,pixel_view->id);
   pixel_view->exception=AcquireExceptionInfo();
   pixel_view->wand=wand;
-  pixel_view->views=AcquireCacheViewThreadSet(pixel_view->wand->images);
+  pixel_view->view=AcquireCacheView(pixel_view->wand->images);
   pixel_view->region.width=wand->images->columns;
   pixel_view->region.height=wand->images->rows;
-  pixel_view->pixel_wands=AcquirePixelsThreadSet(pixel_view->region.width);
+  pixel_view->number_threads=GetPixelCacheMaximumThreads();
+  pixel_view->pixel_wands=AcquirePixelsThreadSet(pixel_view->region.width,
+    pixel_view->number_threads);
   if (pixel_view->pixel_wands == (PixelWand ***) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       strerror(errno));
@@ -878,13 +904,15 @@ WandExport PixelView *NewPixelViewRegion(MagickWand *wand,const long x,
   (void) FormatMagickString(pixel_view->name,MaxTextExtent,"%s-%lu",
     PixelViewId,pixel_view->id);
   pixel_view->exception=AcquireExceptionInfo();
-  pixel_view->views=AcquireCacheViewThreadSet(pixel_view->wand->images);
+  pixel_view->view=AcquireCacheView(pixel_view->wand->images);
   pixel_view->wand=wand;
   pixel_view->region.width=width;
   pixel_view->region.height=height;
   pixel_view->region.x=x;
   pixel_view->region.y=y;
-  pixel_view->pixel_wands=AcquirePixelsThreadSet(pixel_view->region.width);
+  pixel_view->number_threads=GetPixelCacheMaximumThreads();
+  pixel_view->pixel_wands=AcquirePixelsThreadSet(pixel_view->region.width,
+    pixel_view->number_threads);
   if (pixel_view->pixel_wands == (PixelWand ***) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       strerror(errno));
@@ -960,9 +988,14 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
   status=MagickTrue;
   progress=0;
   exception=destination->exception;
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+#endif
   for (y=destination->region.y; y < (long) destination->region.height; y++)
   {
+    MagickBooleanType
+      sync;
+
     register IndexPacket
       *indexes;
 
@@ -975,17 +1008,17 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    pixels=GetCacheViewAuthenticPixels(destination->views[id],destination->region.x,y,
-      destination->region.width,1,exception);
+    id=GetPixelCacheThreadId();
+    pixels=GetCacheViewAuthenticPixels(destination->view,destination->region.x,
+      y,destination->region.width,1,exception);
     if (pixels == (PixelPacket *) NULL)
       {
         InheritException(destination->exception,GetCacheViewException(
-          destination->views[id]));
+          destination->view));
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(destination->views[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(destination->view);
     if (set(destination,context) == MagickFalse)
       status=MagickFalse;
     for (x=0; x < (long) destination->region.width; x++)
@@ -993,10 +1026,11 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
     if (destination_image->colorspace == CMYKColorspace)
       for (x=0; x < (long) destination->region.width; x++)
         indexes[x]=PixelGetBlackQuantum(destination->pixel_wands[id][x]);
-    if (SyncCacheViewAuthenticPixels(destination->views[id],exception) == MagickFalse)
+    sync=SyncCacheViewAuthenticPixels(destination->view,exception);
+    if (sync == MagickFalse)
       {
         InheritException(destination->exception,GetCacheViewException(
-          destination->views[id]));
+          destination->view));
         status=MagickFalse;
       }
     if (destination_image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1004,7 +1038,9 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(destination_image,SetPixelViewTag,progress++,
           destination->region.height);
         if (proceed == MagickFalse)
@@ -1086,9 +1122,14 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
   status=MagickTrue;
   progress=0;
   exception=destination->exception;
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+#endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
+    MagickBooleanType
+      sync;
+
     register const IndexPacket
       *indexes;
 
@@ -1107,15 +1148,15 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    pixels=GetCacheViewVirtualPixels(source->views[id],source->region.x,y,
+    id=GetPixelCacheThreadId();
+    pixels=GetCacheViewVirtualPixels(source->view,source->region.x,y,
       source->region.width,1,source->exception);
     if (pixels == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(source->views[id]);
+    indexes=GetCacheViewVirtualIndexQueue(source->view);
     for (x=0; x < (long) source->region.width; x++)
       PixelSetQuantumColor(source->pixel_wands[id][x],pixels+x);
     if (source_image->colorspace == CMYKColorspace)
@@ -1124,14 +1165,14 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
     if (source_image->storage_class == PseudoClass)
       for (x=0; x < (long) source->region.width; x++)
         PixelSetIndex(source->pixel_wands[id][x],indexes[x]);
-    destination_pixels=GetCacheViewAuthenticPixels(destination->views[id],
+    destination_pixels=GetCacheViewAuthenticPixels(destination->view,
       destination->region.x,y,destination->region.width,1,exception);
     if (destination_pixels == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    destination_indexes=GetCacheViewAuthenticIndexes(destination->views[id]);
+    destination_indexes=GetCacheViewAuthenticIndexQueue(destination->view);
     for (x=0; x < (long) destination->region.width; x++)
       PixelSetQuantumColor(destination->pixel_wands[id][x],pixels+x);
     if (destination_image->colorspace == CMYKColorspace)
@@ -1149,10 +1190,11 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
       for (x=0; x < (long) destination->region.width; x++)
         destination_indexes[x]=PixelGetBlackQuantum(
           destination->pixel_wands[id][x]);
-    if (SyncCacheViewAuthenticPixels(destination->views[id],exception) == MagickFalse)
+    sync=SyncCacheViewAuthenticPixels(destination->view,exception);
+    if (sync == MagickFalse)
       {
         InheritException(destination->exception,GetCacheViewException(
-          source->views[id]));
+          source->view));
         status=MagickFalse;
       }
     if (source_image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1160,7 +1202,9 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(source_image,TransferPixelViewTag,progress++,
           source->region.height);
         if (proceed == MagickFalse)
@@ -1236,7 +1280,9 @@ WandExport MagickBooleanType UpdatePixelViewIterator(PixelView *source,
   status=MagickTrue;
   progress=0;
   exception=source->exception;
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+#endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
     register IndexPacket
@@ -1251,17 +1297,17 @@ WandExport MagickBooleanType UpdatePixelViewIterator(PixelView *source,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    pixels=GetCacheViewAuthenticPixels(source->views[id],source->region.x,y,
+    id=GetPixelCacheThreadId();
+    pixels=GetCacheViewAuthenticPixels(source->view,source->region.x,y,
       source->region.width,1,exception);
     if (pixels == (PixelPacket *) NULL)
       {
         InheritException(source->exception,GetCacheViewException(
-          source->views[id]));
+          source->view));
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(source->views[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(source->view);
     for (x=0; x < (long) source->region.width; x++)
       PixelSetQuantumColor(source->pixel_wands[id][x],pixels+x);
     if (source_image->colorspace == CMYKColorspace)
@@ -1274,10 +1320,9 @@ WandExport MagickBooleanType UpdatePixelViewIterator(PixelView *source,
     if (source_image->colorspace == CMYKColorspace)
       for (x=0; x < (long) source->region.width; x++)
         indexes[x]=PixelGetBlackQuantum(source->pixel_wands[id][x]);
-    if (SyncCacheViewAuthenticPixels(source->views[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(source->view,exception) == MagickFalse)
       {
-        InheritException(source->exception,GetCacheViewException(
-          source->views[id]));
+        InheritException(source->exception,GetCacheViewException(source->view));
         status=MagickFalse;
       }
     if (source_image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1285,7 +1330,9 @@ WandExport MagickBooleanType UpdatePixelViewIterator(PixelView *source,
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(source_image,UpdatePixelViewTag,progress++,
           source->region.height);
         if (proceed == MagickFalse)
