@@ -10,14 +10,14 @@
 %                         W   W  A   A  N   N  DDDD                           %
 %                                                                             %
 %                                                                             %
-%                           Wand Support Methods                              %
+%                         MagickWand Support Methods                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 May  2004                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -70,7 +70,6 @@ static SemaphoreInfo
 %
 %      unsigned long AcquireWandId()
 %
-%
 */
 WandExport unsigned long AcquireWandId(void)
 {
@@ -86,16 +85,14 @@ WandExport unsigned long AcquireWandId(void)
         {
           wand_ids=NewSplayTree((int (*)(const void *,const void *)) NULL,
             (void *(*)(void *)) NULL,(void *(*)(void *)) NULL);
-          if (IsMagickInstantiated() == MagickFalse)
-            MagickCoreGenesis((char *) NULL,MagickFalse);
           instantiate_wand=MagickTrue;
         }
       RelinquishSemaphoreInfo(wand_semaphore);
     }
   AcquireSemaphoreInfo(&wand_semaphore);
   id++;
-  RelinquishSemaphoreInfo(wand_semaphore);
   (void) AddValueToSplayTree(wand_ids,(const void *) id,(const void *) id);
+  RelinquishSemaphoreInfo(wand_semaphore);
   return(id);
 }
 
@@ -126,14 +123,16 @@ WandExport void RelinquishWandId(const unsigned long id)
   (void) LogMagickEvent(WandEvent,GetMagickModule(),"...");
   if (wand_ids == (SplayTreeInfo *) NULL)
     return;
-  (void) RemoveNodeByValueFromSplayTree(wand_ids,(const void *) id);
+  AcquireSemaphoreInfo(&wand_semaphore);
+  (void) DeleteNodeByValueFromSplayTree(wand_ids,(const void *) id);
   if ((instantiate_wand == MagickFalse) ||
       (GetNumberOfNodesInSplayTree(wand_ids) != 0))
-    return;
-  AcquireSemaphoreInfo(&wand_semaphore);
-  MagickCoreTerminus();
+    {
+      RelinquishSemaphoreInfo(wand_semaphore);
+      return;
+    }
   wand_ids=DestroySplayTree(wand_ids);
   instantiate_wand=MagickFalse;
   RelinquishSemaphoreInfo(wand_semaphore);
-  wand_semaphore=DestroySemaphoreInfo(wand_semaphore);
+  DestroySemaphoreInfo(&wand_semaphore);
 }
